@@ -90,7 +90,9 @@ class IndexedDBBrowserTest : public ContentBrowserTest,
         failure_class, failure_method, fail_on_instance_num, fail_on_call_num);
   }
 
-  void SimpleTest(const GURL& test_url, bool incognito = false) {
+  void SimpleTest(const GURL& test_url,
+                  bool incognito = false,
+                  Shell** shell_out = nullptr) {
     // The test page will perform tests on IndexedDB, then navigate to either
     // a #pass or #fail ref.
     Shell* the_browser = incognito ? CreateOffTheRecordBrowser() : shell();
@@ -107,6 +109,8 @@ class IndexedDBBrowserTest : public ContentBrowserTest,
           &js_result));
       FAIL() << "Failed: " << js_result;
     }
+    if (shell_out)
+      *shell_out = the_browser;
   }
 
   void NavigateAndWaitForTitle(Shell* shell,
@@ -307,6 +311,21 @@ IN_PROC_BROWSER_TEST_F(IndexedDBBrowserTest, Bug109187Test) {
   NavigateToURLBlockUntilNavigationsComplete(shell(), url, 1);
 }
 
+IN_PROC_BROWSER_TEST_F(IndexedDBBrowserTest, Bug941965Test) {
+  // Double-open an incognito window to test that saving & reading a blob from
+  // indexeddb works.
+  Shell* incognito_browser = nullptr;
+  SimpleTest(GetTestUrl("indexeddb", "simple_blob_read.html"), true,
+             &incognito_browser);
+  ASSERT_TRUE(incognito_browser);
+  incognito_browser->Close();
+  incognito_browser = nullptr;
+  SimpleTest(GetTestUrl("indexeddb", "simple_blob_read.html"), true,
+             &incognito_browser);
+  ASSERT_TRUE(incognito_browser);
+  incognito_browser->Close();
+}
+
 class IndexedDBBrowserTestWithLowQuota : public IndexedDBBrowserTest {
  public:
   IndexedDBBrowserTestWithLowQuota() {}
@@ -336,6 +355,20 @@ class IndexedDBBrowserTestWithGCExposed : public IndexedDBBrowserTest {
   DISALLOW_COPY_AND_ASSIGN(IndexedDBBrowserTestWithGCExposed);
 };
 
+class IndexedDBBrowserTestWithExperimentalWebFeatures
+    : public IndexedDBBrowserTest {
+ public:
+  IndexedDBBrowserTestWithExperimentalWebFeatures() = default;
+
+  void SetUpCommandLine(base::CommandLine* command_line) override {
+    command_line->AppendSwitch(
+        switches::kEnableExperimentalWebPlatformFeatures);
+  }
+
+ private:
+  DISALLOW_COPY_AND_ASSIGN(IndexedDBBrowserTestWithExperimentalWebFeatures);
+};
+
 IN_PROC_BROWSER_TEST_F(IndexedDBBrowserTestWithGCExposed,
                        DatabaseCallbacksTest) {
   SimpleTest(GetTestUrl("indexeddb", "database_callbacks_first.html"));
@@ -447,6 +480,39 @@ IN_PROC_BROWSER_TEST_F(IndexedDBBrowserTestWithMissingSSTFile,
   EXPECT_NE(original_size, new_size);
 }
 
+// IndexedDBBrowserTestWithCrbug899446* capture IDB instances from Chrome stable
+// to verify that the current code can read those instances.  For more info on
+// a case when Chrome canary couldn't read stable's IDB instances, see
+// https://crbug.com/899446.
+
+class IndexedDBBrowserTestWithCrbug899446
+    : public IndexedDBBrowserTestWithPreexistingLevelDB {
+  std::string EnclosingLevelDBDir() override { return "crbug899446"; }
+};
+
+IN_PROC_BROWSER_TEST_F(IndexedDBBrowserTestWithCrbug899446, StableTest) {
+  int64_t original_size = RequestUsage(kFileOrigin);
+  EXPECT_GT(original_size, 0);
+  SimpleTest(GetTestUrl("indexeddb", "crbug899446.html"));
+  int64_t new_size = RequestUsage(kFileOrigin);
+  EXPECT_GT(new_size, 0);
+  EXPECT_NE(original_size, new_size);
+}
+
+class IndexedDBBrowserTestWithCrbug899446Noai
+    : public IndexedDBBrowserTestWithPreexistingLevelDB {
+  std::string EnclosingLevelDBDir() override { return "crbug899446_noai"; }
+};
+
+IN_PROC_BROWSER_TEST_F(IndexedDBBrowserTestWithCrbug899446Noai, StableTest) {
+  int64_t original_size = RequestUsage(kFileOrigin);
+  EXPECT_GT(original_size, 0);
+  SimpleTest(GetTestUrl("indexeddb", "crbug899446_noai.html"));
+  int64_t new_size = RequestUsage(kFileOrigin);
+  EXPECT_GT(new_size, 0);
+  EXPECT_NE(original_size, new_size);
+}
+
 IN_PROC_BROWSER_TEST_F(IndexedDBBrowserTest, LevelDBLogFileTest) {
   // Any page that opens an IndexedDB will work here.
   SimpleTest(GetTestUrl("indexeddb", "database_test.html"));
@@ -892,6 +958,34 @@ IN_PROC_BROWSER_TEST_F(
   EXPECT_EQ(expected_title16, title_watcher.WaitAndGetTitle());
 }
 
+// Testing abort ordering for commit. Verifies that an explicitly committed
+// transaction blocked by another pending transaction will be committed rather
+// than aborted in the event that the page crashes before its committed.
+IN_PROC_BROWSER_TEST_F(IndexedDBBrowserTestWithExperimentalWebFeatures,
+                       CommitContinuesOnAbort) {
+  // Sets up a database, opens four transactions against it such that two end
+  // up indefinitely blocked, one of which is explicitly committed.
+  NavigateAndWaitForTitle(shell(), "blocked_explicit_commit.html", "#tab1",
+                          "transactions registered");
+
+  // Crashes the tab to cause the database set up above to force close with the
+  // blocked transactions still open.
+  shell()->web_contents()->GetMainFrame()->GetProcess()->Shutdown(0);
+
+  // Reopens the same page that was just crashed and inspects the database to
+  // see the results of the transactions that were open at time of crash.
+  Shell* new_shell = CreateBrowser();
+  GURL url = GetTestUrl("indexeddb", "blocked_explicit_commit.html");
+  url = GURL(url.spec() + "#tab2");
+  base::string16 expected_title16(
+      ASCIIToUTF16("transactions aborted and committed as expected"));
+  TitleWatcher title_watcher(new_shell->web_contents(), expected_title16);
+  title_watcher.AlsoWaitForTitle(
+      ASCIIToUTF16("fail - transactions did not abort and commit as expected"));
+  NavigateToURL(new_shell, url);
+  EXPECT_EQ(expected_title16, title_watcher.WaitAndGetTitle());
+}
+
 // Verify that a "close" event is fired at database connections when
 // the backing store is deleted.
 IN_PROC_BROWSER_TEST_F(IndexedDBBrowserTest, ForceCloseEventTest) {
