@@ -8,6 +8,7 @@
 #include "content/browser/media/session/media_session_controller.h"
 #include "content/browser/media/session/media_session_impl.h"
 #include "content/common/media/media_player_delegate_messages.h"
+#include "content/public/test/test_service_manager_context.h"
 #include "content/test/test_render_view_host.h"
 #include "content/test/test_web_contents.h"
 #include "testing/gtest/include/gtest/gtest.h"
@@ -18,6 +19,10 @@ class MediaSessionControllerTest : public RenderViewHostImplTestHarness {
  public:
   void SetUp() override {
     RenderViewHostImplTestHarness::SetUp();
+
+    test_service_manager_context_ =
+        std::make_unique<content::TestServiceManagerContext>();
+
     id_ = WebContentsObserver::MediaPlayerId(contents()->GetMainFrame(), 0);
     controller_ = CreateController();
   }
@@ -26,6 +31,8 @@ class MediaSessionControllerTest : public RenderViewHostImplTestHarness {
     // Destruct the controller prior to any other teardown to avoid out of order
     // destruction relative to the MediaSession instance.
     controller_.reset();
+
+    test_service_manager_context_.reset();
     RenderViewHostImplTestHarness::TearDown();
   }
 
@@ -120,6 +127,10 @@ class MediaSessionControllerTest : public RenderViewHostImplTestHarness {
   WebContentsObserver::MediaPlayerId id_ =
       WebContentsObserver::MediaPlayerId::createMediaPlayerIdForTests();
   std::unique_ptr<MediaSessionController> controller_;
+
+ private:
+  std::unique_ptr<content::TestServiceManagerContext>
+      test_service_manager_context_;
 };
 
 TEST_F(MediaSessionControllerTest, NoAudioNoSession) {
