@@ -15,6 +15,7 @@
 #include "base/test/metrics/histogram_tester.h"
 #include "base/test/scoped_task_environment.h"
 #include "base/threading/thread_task_runner_handle.h"
+#include "build/build_config.h"
 #include "content/browser/renderer_host/input/timeout_monitor.h"
 #include "content/common/input/synthetic_web_input_event_builders.h"
 #include "testing/gtest/include/gtest/gtest.h"
@@ -126,11 +127,14 @@ const float kWheelScrollGlobalY = 72;
   EXPECT_EQ(WebGestureEvent::kMomentumPhase,      \
             event->data.scroll_end.inertial_phase);
 
+#if defined(CHROME_OS)
 #define EXPECT_SYNTHETIC_INERTIAL_GESTURE_SCROLL_END(event) \
   EXPECT_GESTURE_SCROLL_END_IMPL(event);                    \
   EXPECT_TRUE(event->data.scroll_end.synthetic);            \
   EXPECT_EQ(WebGestureEvent::kMomentumPhase,                \
-            event->data.scroll_end.inertial_phase);
+            event->data.scroll_end.inertial_phase);         \
+  EXPECT_TRUE(event->data.scroll_end.generated_by_fling_controller);
+#endif
 
 #define EXPECT_MOUSE_WHEEL(event) \
   EXPECT_EQ(WebInputEvent::kMouseWheel, event->GetType());
@@ -220,21 +224,23 @@ class MouseWheelEventQueueTest : public testing::Test,
                                  ack_result, ui::LatencyInfo());
   }
   void SendMouseWheel(float x,
-                               float y,
-                               float global_x,
-                               float global_y,
-                               float dX,
-                               float dY,
-                               int modifiers,
-                               bool high_precision,
-                               blink::WebMouseWheelEvent::Phase phase,
-                               blink::WebMouseWheelEvent::Phase momentum_phase,
-                               WebInputEvent::RailsMode rails_mode) {
+                      float y,
+                      float global_x,
+                      float global_y,
+                      float dX,
+                      float dY,
+                      int modifiers,
+                      bool high_precision,
+                      blink::WebMouseWheelEvent::Phase phase,
+                      blink::WebMouseWheelEvent::Phase momentum_phase,
+                      WebInputEvent::RailsMode rails_mode,
+                      bool has_synthetic_phase = false) {
     WebMouseWheelEvent event = SyntheticWebMouseWheelEventBuilder::Build(
         x, y, global_x, global_y, dX, dY, modifiers, high_precision);
     event.phase = phase;
     event.momentum_phase = momentum_phase;
     event.rails_mode = rails_mode;
+    event.has_synthetic_phase = has_synthetic_phase;
     queue_->QueueEvent(MouseWheelEventWithLatencyInfo(event));
   }
   void SendMouseWheel(float x,
@@ -246,9 +252,11 @@ class MouseWheelEventQueueTest : public testing::Test,
                       int modifiers,
                       bool high_precision,
                       blink::WebMouseWheelEvent::Phase phase,
-                      blink::WebMouseWheelEvent::Phase momentum_phase) {
+                      blink::WebMouseWheelEvent::Phase momentum_phase,
+                      bool has_synthetic_phase = false) {
     SendMouseWheel(x, y, global_x, global_y, dX, dY, modifiers, high_precision,
-                   phase, momentum_phase, WebInputEvent::kRailsModeFree);
+                   phase, momentum_phase, WebInputEvent::kRailsModeFree,
+                   has_synthetic_phase);
   }
 
   void SendGestureEvent(WebInputEvent::Type type) {
@@ -426,6 +434,40 @@ TEST_F(MouseWheelEventQueueTest,
   PhaseGestureSendingTest(true);
 }
 
+// Tests that a Wheel event with synthetic momentumn_phase == PhaseEnded that is
+// generated by the fling controller properly populates
+// scroll_end.data.scroll_end.generated_by_fling_controller.
+#if defined(CHROME_OS)
+TEST_F(MouseWheelEventQueueTest, WheelEndWithMomentumPhaseEndedInformation) {
+  const WebGestureEvent::ScrollUnits scroll_units =
+      WebGestureEvent::kPrecisePixels;
+  SendMouseWheel(kWheelScrollX, kWheelScrollY, kWheelScrollGlobalX,
+                 kWheelScrollGlobalY, 1, 1, 0, true /* high_precision */,
+                 WebMouseWheelEvent::kPhaseBegan,
+                 WebMouseWheelEvent::kPhaseNone);
+  EXPECT_EQ(0U, queued_event_count());
+  EXPECT_TRUE(event_in_flight());
+  EXPECT_EQ(1U, GetAndResetSentEventCount());
+  SendMouseWheelEventAck(INPUT_EVENT_ACK_STATE_NOT_CONSUMED);
+  EXPECT_GESTURE_SCROLL_BEGIN_WITH_PHASE(sent_gesture_event(0));
+  EXPECT_GESTURE_SCROLL_UPDATE_WITH_PHASE(sent_gesture_event(1));
+  EXPECT_EQ(2U, GetAndResetSentEventCount());
+
+  SendMouseWheel(kWheelScrollX, kWheelScrollY, kWheelScrollGlobalX,
+                 kWheelScrollGlobalY, 0, 0, 0, true /* high_precision */,
+                 WebMouseWheelEvent::kPhaseNone,
+                 WebMouseWheelEvent::kPhaseEnded, true /*has_synthetic_phase*/);
+  EXPECT_EQ(0U, queued_event_count());
+  EXPECT_TRUE(event_in_flight());
+  EXPECT_EQ(1U, GetAndResetSentEventCount());
+  SendMouseWheelEventAck(INPUT_EVENT_ACK_STATE_NOT_CONSUMED);
+
+  EXPECT_EQ(1U, all_sent_events().size());
+  EXPECT_SYNTHETIC_INERTIAL_GESTURE_SCROLL_END(sent_gesture_event(0));
+  EXPECT_EQ(1U, GetAndResetSentEventCount());
+}
+#endif  // defined(CHROME_OS)
+
 TEST_F(MouseWheelEventQueueTest, GestureSendingInterrupted) {
   const WebGestureEvent::ScrollUnits scroll_units = WebGestureEvent::kPixels;
   SendMouseWheel(kWheelScrollX, kWheelScrollY, kWheelScrollGlobalX,
@@ -621,4 +663,29 @@ TEST_F(MouseWheelEventQueueTest, WheelScrollingWasLatchedHistogramCheck) {
     histogram_tester.ExpectBucketCount(latching_histogram_name, 1, 1);
 }
 
+#if defined(OS_MACOSX)
+TEST_F(MouseWheelEventQueueTest, DoNotSwapXYForShiftScroll) {
+  // Send an event with shift modifier, zero value for delta X, and no direction
+  // for |rails_mode|. Do not swap the scroll direction.
+  SendMouseWheel(kWheelScrollX, kWheelScrollY, kWheelScrollGlobalX,
+                 kWheelScrollGlobalY, 0.0, 1.0, WebInputEvent::kShiftKey, false,
+                 WebMouseWheelEvent::kPhaseBegan,
+                 WebMouseWheelEvent::kPhaseNone, WebInputEvent::kRailsModeFree);
+  EXPECT_EQ(0U, queued_event_count());
+  EXPECT_TRUE(event_in_flight());
+  EXPECT_EQ(1U, GetAndResetSentEventCount());
+
+  // Receive an ACK for the mouse wheel event.
+  SendMouseWheelEventAck(INPUT_EVENT_ACK_STATE_NOT_CONSUMED);
+  EXPECT_EQ(0U, queued_event_count());
+  EXPECT_FALSE(event_in_flight());
+  EXPECT_EQ(WebInputEvent::kMouseWheel, acked_event().GetType());
+  EXPECT_EQ(1U, GetAndResetAckedEventCount());
+
+  EXPECT_EQ(2U, all_sent_events().size());
+  EXPECT_EQ(0U, sent_gesture_event(1)->data.scroll_update.delta_x);
+  EXPECT_EQ(1U, sent_gesture_event(1)->data.scroll_update.delta_y);
+  EXPECT_EQ(2U, GetAndResetSentEventCount());
+}
+#endif
 }  // namespace content
