@@ -16,6 +16,7 @@
 #include <utility>
 #include <vector>
 
+#include "base/callback_forward.h"
 #include "base/containers/flat_set.h"
 #include "base/gtest_prod_util.h"
 #include "base/macros.h"
@@ -36,7 +37,6 @@
 #include "content/common/associated_interfaces.mojom.h"
 #include "content/common/child_control.mojom.h"
 #include "content/common/content_export.h"
-#include "content/common/media/media_stream.mojom.h"
 #include "content/common/media/renderer_audio_output_stream_factory.mojom.h"
 #include "content/common/renderer.mojom.h"
 #include "content/common/renderer_host.mojom.h"
@@ -51,7 +51,9 @@
 #include "mojo/public/cpp/bindings/associated_binding_set.h"
 #include "mojo/public/cpp/bindings/interface_ptr.h"
 #include "mojo/public/cpp/system/invitation.h"
+#include "services/network/public/mojom/mdns_responder.mojom.h"
 #include "services/network/public/mojom/url_loader_factory.mojom.h"
+#include "services/resource_coordinator/public/cpp/process_resource_coordinator.h"
 #include "services/service_manager/public/cpp/binder_registry.h"
 #include "services/service_manager/public/mojom/service.mojom.h"
 #include "services/viz/public/interfaces/compositing/compositing_mode_watcher.mojom.h"
@@ -61,8 +63,9 @@
 #include "third_party/blink/public/mojom/dom_storage/storage_partition_service.mojom.h"
 #include "third_party/blink/public/mojom/filesystem/file_system.mojom.h"
 #include "third_party/blink/public/mojom/indexeddb/indexeddb.mojom.h"
+#include "third_party/blink/public/mojom/mediastream/media_stream.mojom.h"
+#include "third_party/blink/public/platform/modules/broadcastchannel/broadcast_channel.mojom.h"
 #include "ui/gfx/gpu_memory_buffer.h"
-#include "ui/gl/gpu_switching_observer.h"
 
 #if defined(OS_ANDROID)
 #include "content/public/browser/android/child_process_importance.h"
@@ -70,7 +73,6 @@
 
 namespace base {
 class CommandLine;
-class MessageLoop;
 class SharedPersistentMemoryAllocator;
 }
 
@@ -84,6 +86,7 @@ class ChildConnection;
 class FileSystemManagerImpl;
 class IndexedDBDispatcherHost;
 class InProcessChildThreadParams;
+class IsolationContext;
 class MediaStreamTrackMetricsHost;
 class P2PSocketDispatcherHost;
 class PermissionServiceContext;
@@ -93,8 +96,6 @@ class PushMessagingManager;
 class RenderFrameMessageFilter;
 class RenderProcessHostFactory;
 class RenderWidgetHelper;
-class RenderWidgetHost;
-class RenderWidgetHostImpl;
 class ResourceMessageFilter;
 class ServiceWorkerDispatcherHost;
 class SiteInstance;
@@ -128,7 +129,6 @@ typedef base::Thread* (*RendererMainThreadFactoryFunction)(
 class CONTENT_EXPORT RenderProcessHostImpl
     : public RenderProcessHost,
       public ChildProcessLauncher::Client,
-      public ui::GpuSwitchingObserver,
       public mojom::RouteProvider,
       public blink::mojom::AssociatedInterfaceProvider,
       public mojom::RendererHost {
@@ -159,36 +159,39 @@ class CONTENT_EXPORT RenderProcessHostImpl
   void RemoveObserver(RenderProcessHostObserver* observer) override;
   void ShutdownForBadMessage(CrashReportMode crash_report_mode) override;
   void UpdateClientPriority(PriorityClient* client) override;
-  int VisibleClientCount() const override;
-  unsigned int GetFrameDepth() const override;
-  bool GetIntersectsViewport() const override;
-  bool IsForGuestsOnly() const override;
-  StoragePartition* GetStoragePartition() const override;
+  int VisibleClientCount() override;
+  unsigned int GetFrameDepth() override;
+  bool GetIntersectsViewport() override;
+  bool IsForGuestsOnly() override;
+  StoragePartition* GetStoragePartition() override;
   bool Shutdown(int exit_code) override;
   bool FastShutdownIfPossible(size_t page_count = 0,
                               bool skip_unload_handlers = false) override;
-  const base::Process& GetProcess() const override;
-  bool IsReady() const override;
-  BrowserContext* GetBrowserContext() const override;
-  bool InSameStoragePartition(StoragePartition* partition) const override;
-  int GetID() const override;
-  bool IsInitializedAndNotDead() const override;
-  void SetIgnoreInputEvents(bool ignore_input_events) override;
-  bool IgnoreInputEvents() const override;
+  const base::Process& GetProcess() override;
+  bool IsReady() override;
+  BrowserContext* GetBrowserContext() override;
+  bool InSameStoragePartition(StoragePartition* partition) override;
+  int GetID() override;
+  bool IsInitializedAndNotDead() override;
+  void SetBlocked(bool blocked) override;
+  bool IsBlocked() override;
+  std::unique_ptr<base::CallbackList<void(bool)>::Subscription>
+  RegisterBlockStateChangedCallback(
+      const base::RepeatingCallback<void(bool)>& cb) override;
   void Cleanup() override;
   void AddPendingView() override;
   void RemovePendingView() override;
-  void AddWidget(RenderWidgetHost* widget) override;
-  void RemoveWidget(RenderWidgetHost* widget) override;
+  void AddPriorityClient(PriorityClient* priority_client) override;
+  void RemovePriorityClient(PriorityClient* priority_client) override;
 #if defined(OS_ANDROID)
   ChildProcessImportance GetEffectiveImportance() override;
 #endif
   void SetSuddenTerminationAllowed(bool enabled) override;
-  bool SuddenTerminationAllowed() const override;
+  bool SuddenTerminationAllowed() override;
   IPC::ChannelProxy* GetChannel() override;
   void AddFilter(BrowserMessageFilter* filter) override;
-  bool FastShutdownStarted() const override;
-  base::TimeDelta GetChildProcessIdleTime() const override;
+  bool FastShutdownStarted() override;
+  base::TimeDelta GetChildProcessIdleTime() override;
   void FilterURL(bool empty_allowed, GURL* url) override;
   void EnableAudioDebugRecordings(const base::FilePath& file) override;
   void DisableAudioDebugRecordings() override;
@@ -199,14 +202,15 @@ class CONTENT_EXPORT RenderProcessHostImpl
       bool incoming,
       bool outgoing,
       const WebRtcRtpPacketCallback& packet_callback) override;
-  void SetWebRtcEventLogOutput(int lid, bool enabled) override;
+  void EnableWebRtcEventLogOutput(int lid, int output_period_ms) override;
+  void DisableWebRtcEventLogOutput(int lid) override;
   void BindInterface(const std::string& interface_name,
                      mojo::ScopedMessagePipeHandle interface_pipe) override;
-  const service_manager::Identity& GetChildIdentity() const override;
+  const service_manager::Identity& GetChildIdentity() override;
   std::unique_ptr<base::SharedPersistentMemoryAllocator> TakeMetricsAllocator()
       override;
-  const base::TimeTicks& GetInitTimeForNavigationMetrics() const override;
-  bool IsProcessBackgrounded() const override;
+  const base::TimeTicks& GetInitTimeForNavigationMetrics() override;
+  bool IsProcessBackgrounded() override;
   void IncrementKeepAliveRefCount(
       RenderProcessHost::KeepAliveClientType) override;
   void DecrementKeepAliveRefCount(
@@ -219,7 +223,8 @@ class CONTENT_EXPORT RenderProcessHostImpl
   resource_coordinator::ProcessResourceCoordinator*
   GetProcessResourceCoordinator() override;
   void CreateURLLoaderFactory(
-      const url::Origin& origin,
+      const base::Optional<url::Origin>& origin,
+      network::mojom::TrustedURLLoaderHeaderClientPtrInfo header_client,
       network::mojom::URLLoaderFactoryRequest request) override;
 
   void SetIsNeverSuitableForReuse() override;
@@ -228,9 +233,13 @@ class CONTENT_EXPORT RenderProcessHostImpl
   void SetIsUsed() override;
 
   bool HostHasNotBeenUsed() override;
-  void LockToOrigin(const GURL& lock_url) override;
+  void LockToOrigin(const IsolationContext& isolation_context,
+                    const GURL& lock_url) override;
   void BindCacheStorage(blink::mojom::CacheStorageRequest request,
                         const url::Origin& origin) override;
+  void BindIndexedDB(blink::mojom::IDBFactoryRequest request,
+                     const url::Origin& origin) override;
+  void ForceCrash() override;
   void CleanupCorbExceptionForPluginUponDestruction() override;
 
   mojom::RouteProvider* GetRemoteRouteProvider();
@@ -282,6 +291,7 @@ class CONTENT_EXPORT RenderProcessHostImpl
   // hosted apps.
   static bool IsSuitableHost(RenderProcessHost* host,
                              BrowserContext* browser_context,
+                             const IsolationContext& isolation_context,
                              const GURL& site_url,
                              const GURL& lock_url);
 
@@ -294,12 +304,14 @@ class CONTENT_EXPORT RenderProcessHostImpl
   // Important: |url| should be a full URL and *not* a site URL.
   static RenderProcessHost* GetSoleProcessHostForURL(
       BrowserContext* browser_context,
+      const IsolationContext& isolation_context,
       const GURL& url);
 
   // Variant of the above that takes in a SiteInstance site URL and the
   // process's origin lock URL, when they are known.
   static RenderProcessHost* GetSoleProcessHostForSite(
       BrowserContext* browser_context,
+      const IsolationContext& isolation_context,
       const GURL& site_url,
       const GURL& lock_url);
 
@@ -352,7 +364,13 @@ class CONTENT_EXPORT RenderProcessHostImpl
     kMaxValue = kRefusedBySiteInstance
   };
 
-  static base::MessageLoop* GetInProcessRendererThreadForTesting();
+  static scoped_refptr<base::SingleThreadTaskRunner>
+  GetInProcessRendererThreadTaskRunnerForTesting();
+
+#if !defined(OS_ANDROID) && !defined(OS_CHROMEOS)
+  // Gets the platform-specific limit. Used by GetMaxRendererProcessCount().
+  static size_t GetPlatformMaxRendererProcessCount();
+#endif
 
   // This forces a renderer that is running "in process" to shut down.
   static void ShutDownInProcessRenderer();
@@ -360,14 +378,23 @@ class CONTENT_EXPORT RenderProcessHostImpl
   static void RegisterRendererMainThreadFactory(
       RendererMainThreadFactoryFunction create);
 
-  // Allows external code to supply a function which creates a
-  // StoragePartitionService. Used for supplying test versions of the
+  // Allows external code to supply a callback which handles a
+  // StoragePartitionServiceRequest. Used for supplying test versions of the
   // service.
-  using CreateStoragePartitionServiceFunction =
-      void (*)(RenderProcessHostImpl* rph,
-               blink::mojom::StoragePartitionServiceRequest request);
-  static void SetCreateStoragePartitionServiceFunction(
-      CreateStoragePartitionServiceFunction function);
+  using StoragePartitionServiceRequestHandler = base::RepeatingCallback<void(
+      RenderProcessHostImpl* rph,
+      blink::mojom::StoragePartitionServiceRequest request)>;
+  static void SetStoragePartitionServiceRequestHandlerForTesting(
+      StoragePartitionServiceRequestHandler handler);
+
+  // Allows external code to supply a callback which handles a
+  // BroadcastChannelProviderRequest. Used for supplying test versions of the
+  // service.
+  using BroadcastChannelProviderRequestHandler = base::RepeatingCallback<void(
+      RenderProcessHostImpl* rph,
+      blink::mojom::BroadcastChannelProviderRequest request)>;
+  static void SetBroadcastChannelProviderRequestHandlerForTesting(
+      BroadcastChannelProviderRequestHandler handler);
 
   RenderFrameMessageFilter* render_frame_message_filter_for_testing() const {
     return render_frame_message_filter_.get();
@@ -385,8 +412,6 @@ class CONTENT_EXPORT RenderProcessHostImpl
   static void EarlyZygoteLaunch();
 #endif  // defined(OS_POSIX) && !defined(OS_ANDROID) && !defined(OS_MACOSX)
 
-  void RecomputeAndUpdateWebKitPreferences();
-
   RendererAudioOutputStreamFactoryContext*
   GetRendererAudioOutputStreamFactoryContext() override;
 
@@ -396,6 +421,9 @@ class CONTENT_EXPORT RenderProcessHostImpl
   void OnMediaStreamRemoved() override;
   int get_media_stream_count_for_testing() const { return media_stream_count_; }
 
+  void OnForegroundServiceWorkerAdded() override;
+  void OnForegroundServiceWorkerRemoved() override;
+
   // Sets the global factory used to create new RenderProcessHosts in unit
   // tests.  It may be nullptr, in which case the default RenderProcessHost will
   // be created (this is the behavior if you don't call this function).  The
@@ -534,6 +562,8 @@ class CONTENT_EXPORT RenderProcessHostImpl
       viz::mojom::CompositingModeReporterRequest request);
   void CreateStoragePartitionService(
       blink::mojom::StoragePartitionServiceRequest request);
+  void CreateBroadcastChannelProvider(
+      blink::mojom::BroadcastChannelProviderRequest request);
   void CreateRendererHost(mojom::RendererHostAssociatedRequest request);
   void BindVideoDecoderService(media::mojom::InterfaceFactoryRequest request);
 
@@ -575,9 +605,6 @@ class CONTENT_EXPORT RenderProcessHostImpl
   // any other that hang off it.
   void ResetIPC();
 
-  // GpuSwitchingObserver implementation.
-  void OnGpuSwitched() override;
-
   void RecordKeepAliveDuration(RenderProcessHost::KeepAliveClientType,
                                base::TimeTicks start,
                                base::TimeTicks end);
@@ -601,9 +628,13 @@ class CONTENT_EXPORT RenderProcessHostImpl
 
   void CreateMediaStreamDispatcherHost(
       MediaStreamManager* media_stream_manager,
-      mojom::MediaStreamDispatcherHostRequest request);
+      blink::mojom::MediaStreamDispatcherHostRequest request);
   void CreateMediaStreamTrackMetricsHost(
-      mojom::MediaStreamTrackMetricsHostRequest request);
+      blink::mojom::MediaStreamTrackMetricsHostRequest request);
+
+#if BUILDFLAG(ENABLE_MDNS)
+  void CreateMdnsResponder(network::mojom::MdnsResponderRequest request);
+#endif  // BUILDFLAG(ENABLE_MDNS)
 
   void OnRegisterAecDumpConsumer(int id);
   void OnUnregisterAecDumpConsumer(int id);
@@ -656,6 +687,15 @@ class CONTENT_EXPORT RenderProcessHostImpl
   // execute.
   void CancelProcessShutdownDelayForUnload();
 
+  // Creates a URLLoaderFactory that can be used by the renderer process,
+  // without binding it to a specific frame or an origin.
+  //
+  // TODO(kinuko, lukasza): https://crbug.com/891872: Remove, once all
+  // URLLoaderFactories are associated with a specific origin and an execution
+  // context (e.g. a frame, a service worker or any other kind of worker).
+  void CreateURLLoaderFactoryForRendererProcess(
+      network::mojom::URLLoaderFactoryRequest request);
+
   mojo::OutgoingInvitation mojo_invitation_;
 
   std::unique_ptr<ChildConnection> child_connection_;
@@ -712,12 +752,9 @@ class CONTENT_EXPORT RenderProcessHostImpl
   ChildProcessImportance effective_importance_ = ChildProcessImportance::NORMAL;
 #endif
 
-  // Clients that contribute priority to this proces.
+  // Clients that contribute priority to this process.
   base::flat_set<PriorityClient*> priority_clients_;
 
-  // The set of widgets in this RenderProcessHostImpl.
-  std::set<RenderWidgetHostImpl*> widgets_;
-
   ChildProcessLauncherPriority priority_;
 
   // Used to allow a RenderWidgetHost to intercept various messages on the
@@ -748,17 +785,15 @@ class CONTENT_EXPORT RenderProcessHostImpl
   // The globally-unique identifier for this RPH.
   const int id_;
 
-  // A secondary ID used by the Service Manager to distinguish different
-  // incarnations of the same RPH from each other. Unlike |id_| this is not
-  // globally unique, but it is guaranteed to change every time ProcessDied() is
-  // called.
-  int instance_id_ = 1;
-
   BrowserContext* const browser_context_;
 
   // Owned by |browser_context_|.
   StoragePartitionImpl* storage_partition_impl_;
 
+  // Keeps track of the BindingIds  returned by storage_partition_impl_->Bind()
+  // calls so we can Unbind() them on cleanup.
+  std::set<mojo::BindingId> storage_partition_binding_ids_;
+
   // The observers watching our lifetime.
   base::ObserverList<RenderProcessHostObserver>::Unchecked observers_;
 
@@ -770,9 +805,12 @@ class CONTENT_EXPORT RenderProcessHostImpl
   // the same process doesn't.
   bool sudden_termination_allowed_;
 
-  // Set to true if we shouldn't send input events.  We actually do the
-  // filtering for this at the render widget level.
-  bool ignore_input_events_;
+  // Set to true if this process is blocked and shouldn't be sent input events.
+  // The checking of this actually happens in the RenderWidgetHost.
+  bool is_blocked_;
+
+  // The clients who want to know when the blocked state has changed.
+  base::CallbackList<void(bool)> blocked_state_changed_callback_list_;
 
   // Records the last time we regarded the child process active.
   base::TimeTicks child_process_activity_time_;
@@ -786,10 +824,6 @@ class CONTENT_EXPORT RenderProcessHostImpl
   // that has a site assigned.
   bool is_unused_;
 
-  // Prevents the class from being added as a GpuDataManagerImpl observer more
-  // than once.
-  bool gpu_observer_registered_;
-
   // Set if a call to Cleanup is required once the RenderProcessHostImpl is no
   // longer within the RenderProcessHostObserver::RenderProcessExited callbacks.
   bool delayed_cleanup_needed_;
@@ -845,15 +879,6 @@ class CONTENT_EXPORT RenderProcessHostImpl
   bool channel_connected_;
   bool sent_render_process_ready_;
 
-#if defined(OS_ANDROID)
-  // UI thread is the source of sync IPCs and all shutdown signals.
-  // Therefore a proper shutdown event to unblock the UI thread is not
-  // possible without massive refactoring shutdown code.
-  // Luckily Android never performs a clean shutdown. So explicitly
-  // ignore this problem.
-  base::WaitableEvent never_signaled_;
-#endif
-
   scoped_refptr<ResourceMessageFilter> resource_message_filter_;
   std::unique_ptr<FileSystemManagerImpl, BrowserThread::DeleteOnIOThread>
       file_system_manager_impl_;
@@ -874,7 +899,12 @@ class CONTENT_EXPORT RenderProcessHostImpl
   // determine if if a process should be backgrounded.
   int media_stream_count_ = 0;
 
-  std::unique_ptr<resource_coordinator::ProcessResourceCoordinator>
+  // Tracks service workers that may need to respond to events from other
+  // processes in a timely manner.  Used to determine if a process should
+  // not be backgrounded.
+  int foreground_service_worker_count_ = 0;
+
+  resource_coordinator::ProcessResourceCoordinator
       process_resource_coordinator_;
 
   // A WeakPtrFactory which is reset every time Cleanup() runs. Used to vend
@@ -891,6 +921,10 @@ class CONTENT_EXPORT RenderProcessHostImpl
   // Fields for recording MediaStream UMA.
   bool has_recorded_media_stream_frame_depth_metric_ = false;
 
+  // If the RenderProcessHost is being shutdown via Shutdown(), this records the
+  // exit code.
+  int shutdown_exit_code_;
+
   base::WeakPtrFactory<RenderProcessHostImpl> weak_factory_;
 
   DISALLOW_COPY_AND_ASSIGN(RenderProcessHostImpl);
