@@ -18,7 +18,6 @@
 #include "content/common/cursors/webcursor.h"
 #include "ipc/ipc_mojo_param_traits.h"
 #include "storage/common/blob_storage/blob_handle.h"
-#include "third_party/blink/public/mojom/choosers/file_chooser.mojom.h"
 #include "third_party/blink/public/platform/web_input_event.h"
 #include "ui/accessibility/ax_mode.h"
 
@@ -34,6 +33,7 @@ struct FrameMsg_ViewChanged_Params;
 namespace viz {
 class FrameSinkId;
 class LocalSurfaceId;
+class LocalSurfaceIdAllocation;
 class SurfaceId;
 class SurfaceInfo;
 }  // namespace viz
@@ -137,6 +137,16 @@ struct CONTENT_EXPORT ParamTraits<viz::LocalSurfaceId> {
   static void Log(const param_type& p, std::string* l);
 };
 
+template <>
+struct CONTENT_EXPORT ParamTraits<viz::LocalSurfaceIdAllocation> {
+  typedef viz::LocalSurfaceIdAllocation param_type;
+  static void Write(base::Pickle* m, const param_type& p);
+  static bool Read(const base::Pickle* m,
+                   base::PickleIterator* iter,
+                   param_type* r);
+  static void Log(const param_type& p, std::string* l);
+};
+
 template <>
 struct CONTENT_EXPORT ParamTraits<viz::SurfaceId> {
   typedef viz::SurfaceId param_type;
@@ -157,17 +167,6 @@ struct CONTENT_EXPORT ParamTraits<viz::SurfaceInfo> {
   static void Log(const param_type& p, std::string* l);
 };
 
-// TODO(tkent): Remove this when we switch to mojo. crbug.com/869257
-template <>
-struct CONTENT_EXPORT ParamTraits<blink::mojom::FileChooserFileInfoPtr> {
-  using param_type = blink::mojom::FileChooserFileInfoPtr;
-  static void Write(base::Pickle* m, const param_type& p);
-  static bool Read(const base::Pickle* m,
-                   base::PickleIterator* iter,
-                   param_type* r);
-  static void Log(const param_type& p, std::string* l);
-};
-
 }  // namespace IPC
 
 #endif  // CONTENT_COMMON_CONTENT_PARAM_TRAITS_H_
