@@ -19,7 +19,7 @@
 #include "base/run_loop.h"
 #include "base/single_thread_task_runner.h"
 #include "base/strings/string_number_conversions.h"
-#include "base/sys_info.h"
+#include "base/system/sys_info.h"
 #include "base/task/post_task.h"
 #include "base/test/test_timeouts.h"
 #include "base/threading/thread_restrictions.h"
@@ -36,6 +36,7 @@
 #include "content/public/browser/web_contents.h"
 #include "content/public/common/content_switches.h"
 #include "content/public/common/main_function_params.h"
+#include "content/public/common/network_service_util.h"
 #include "content/public/common/service_manager_connection.h"
 #include "content/public/common/service_names.mojom.h"
 #include "content/public/test/browser_test_utils.h"
@@ -51,16 +52,26 @@
 #include "services/service_manager/embedder/switches.h"
 #include "services/service_manager/public/cpp/connector.h"
 #include "ui/base/platform_window_defaults.h"
-#include "ui/base/test/material_design_controller_test_api.h"
 #include "ui/compositor/compositor_switches.h"
 #include "ui/display/display_switches.h"
 #include "ui/gl/gl_implementation.h"
 #include "ui/gl/gl_switches.h"
 
+#if defined(OS_MACOSX)
+#include "ui/events/test/event_generator.h"
+#include "ui/views/test/event_generator_delegate_mac.h"
+#endif
+
 #if defined(OS_POSIX)
 #include "base/process/process_handle.h"
 #endif
 
+#if defined(OS_CHROMEOS)
+#include "content/public/browser/network_service_instance.h"
+#include "net/base/network_change_notifier.h"
+#include "net/base/network_change_notifier_posix.h"
+#endif
+
 #if defined(USE_AURA)
 #include "content/browser/compositor/image_transport_factory.h"
 #include "ui/aura/test/event_generator_delegate_aura.h"  // nogncheck
@@ -131,8 +142,7 @@ BrowserTestBase::BrowserTestBase()
       expected_exit_code_(0),
       enable_pixel_output_(false),
       use_software_compositing_(false),
-      set_up_called_(false),
-      disable_io_checks_(false) {
+      set_up_called_(false) {
   ui::test::EnableTestConfigForPlatformWindows();
 
 #if defined(OS_POSIX)
@@ -149,13 +159,16 @@ BrowserTestBase::BrowserTestBase()
 #if defined(USE_AURA)
   ui::test::EventGeneratorDelegate::SetFactoryFunction(base::BindRepeating(
       &aura::test::EventGeneratorDelegateAura::Create, nullptr));
+#elif defined(OS_MACOSX)
+  ui::test::EventGeneratorDelegate::SetFactoryFunction(
+      base::BindRepeating(&views::test::CreateEventGeneratorDelegateMac));
 #endif
 }
 
 BrowserTestBase::~BrowserTestBase() {
 #if defined(OS_ANDROID)
   // RemoteTestServer can cause wait on the UI thread.
-  base::ThreadRestrictions::ScopedAllowWait allow_wait;
+  base::ScopedAllowBaseSyncPrimitivesForTesting allow_wait;
   spawned_test_server_.reset();
 #endif
 
@@ -167,10 +180,6 @@ BrowserTestBase::~BrowserTestBase() {
 
 void BrowserTestBase::SetUp() {
   set_up_called_ = true;
-  // ContentTestSuiteBase might have already initialized
-  // MaterialDesignController in browser_tests suite.
-  // Uninitialize here to let the browser process do it.
-  ui::test::MaterialDesignControllerTestAPI::Uninitialize();
 
   base::CommandLine* command_line = base::CommandLine::ForCurrentProcess();
 
@@ -343,7 +352,7 @@ void BrowserTestBase::SetUp() {
 }
 
 void BrowserTestBase::TearDown() {
-#if defined(USE_AURA)
+#if defined(USE_AURA) || defined(OS_MACOSX)
   ui::test::EventGeneratorDelegate::SetFactoryFunction(
       ui::test::EventGeneratorDelegate::FactoryFunction());
 #endif
@@ -384,6 +393,32 @@ void BrowserTestBase::ProxyRunTestOnMainThreadLoop() {
     signal(SIGTERM, DumpStackTraceSignalHandler);
 #endif  // defined(OS_POSIX)
 
+#if defined(OS_CHROMEOS)
+  // Manually set the connection type since ChromeOS's NetworkChangeNotifier
+  // implementation relies on some other class controlling it (normally
+  // NetworkChangeManagerClient), which may not be set up in all browser tests.
+  net::NetworkChangeNotifierPosix* network_change_notifier =
+      static_cast<net::NetworkChangeNotifierPosix*>(
+          content::GetNetworkChangeNotifier());
+  network_change_notifier->OnConnectionChanged(
+      net::NetworkChangeNotifier::CONNECTION_ETHERNET);
+  // If the network service is enabled, set the connection type for its
+  // NetworkChangeNotifier instance as well.
+  if (base::FeatureList::IsEnabled(network::features::kNetworkService) &&
+      !IsNetworkServiceRunningInProcess()) {
+    network::mojom::NetworkChangeManagerPtr manager_ptr;
+    network::mojom::NetworkChangeManagerRequest request(
+        mojo::MakeRequest(&manager_ptr));
+    GetNetworkService()->GetNetworkChangeManager(std::move(request));
+    manager_ptr->OnNetworkChanged(
+        /*dns_changed=*/false, /*ip_address_changed=*/false,
+        /*connection_type_changed=*/true,
+        network::mojom::ConnectionType::CONNECTION_ETHERNET,
+        /*connection_subtype_changed=*/false,
+        network::mojom::ConnectionSubtype::SUBTYPE_UNKNOWN);
+  }
+#endif
+
   if (base::CommandLine::ForCurrentProcess()->HasSwitch(
           switches::kEnableTracing)) {
     base::trace_event::TraceConfig trace_config(
@@ -423,11 +458,9 @@ void BrowserTestBase::ProxyRunTestOnMainThreadLoop() {
     InitializeNetworkProcess();
 
     bool old_io_allowed_value = false;
-    if (!disable_io_checks_)
-      old_io_allowed_value = base::ThreadRestrictions::SetIOAllowed(false);
+    old_io_allowed_value = base::ThreadRestrictions::SetIOAllowed(false);
     RunTestOnMainThread();
-    if (!disable_io_checks_)
-      base::ThreadRestrictions::SetIOAllowed(old_io_allowed_value);
+    base::ThreadRestrictions::SetIOAllowed(old_io_allowed_value);
     TearDownOnMainThread();
   }
 
@@ -465,16 +498,15 @@ void BrowserTestBase::PostTaskToInProcessRendererAndWait(
   CHECK(base::CommandLine::ForCurrentProcess()->HasSwitch(
       switches::kSingleProcess));
 
-  scoped_refptr<MessageLoopRunner> runner = new MessageLoopRunner;
+  scoped_refptr<base::SingleThreadTaskRunner> renderer_task_runner =
+      RenderProcessHostImpl::GetInProcessRendererThreadTaskRunnerForTesting();
+  CHECK(renderer_task_runner);
 
-  base::MessageLoop* renderer_loop =
-      RenderProcessHostImpl::GetInProcessRendererThreadForTesting();
-  CHECK(renderer_loop);
-
-  renderer_loop->task_runner()->PostTask(
+  base::RunLoop run_loop;
+  renderer_task_runner->PostTask(
       FROM_HERE,
-      base::BindOnce(&RunTaskOnRendererThread, task, runner->QuitClosure()));
-  runner->Run();
+      base::BindOnce(&RunTaskOnRendererThread, task, run_loop.QuitClosure()));
+  run_loop.Run();
 }
 
 void BrowserTestBase::EnablePixelOutput() { enable_pixel_output_ = true; }
@@ -503,10 +535,8 @@ void BrowserTestBase::InitializeNetworkProcess() {
 
   // Send the host resolver rules to the network service if it's in use. No need
   // to do this if it's running in the browser process though.
-  if (!base::FeatureList::IsEnabled(network::features::kNetworkService) ||
-      IsNetworkServiceRunningInProcess()) {
+  if (!IsOutOfProcessNetworkService())
     return;
-  }
 
   net::RuleBasedHostResolverProc::RuleList rules = host_resolver()->GetRules();
   std::vector<network::mojom::RulePtr> mojo_rules;
