@@ -5,6 +5,7 @@
 #include "extensions/shell/test/shell_test.h"
 
 #include "base/command_line.h"
+#include "base/files/file_path.h"
 #include "base/logging.h"
 #include "base/message_loop/message_loop.h"
 #include "base/run_loop.h"
@@ -13,16 +14,22 @@
 #include "extensions/shell/browser/desktop_controller.h"
 #include "extensions/shell/browser/shell_content_browser_client.h"
 #include "extensions/shell/browser/shell_extension_system.h"
+#include "ui/base/ui_base_features.h"
 
 namespace extensions {
 
 AppShellTest::AppShellTest()
     : browser_context_(nullptr),
       extension_system_(nullptr) {
+  // Disable mash until app_shell test infra is updated to use Window Service.
+  scoped_feature_list_.InitWithFeatures(
+      {} /* enabled */,
+      {features::kMash, features::kSingleProcessMash} /* disabled */);
+
+  CreateTestServer(base::FilePath(FILE_PATH_LITERAL("extensions/test/data")));
 }
 
-AppShellTest::~AppShellTest() {
-}
+AppShellTest::~AppShellTest() = default;
 
 void AppShellTest::SetUp() {
   base::CommandLine* command_line = base::CommandLine::ForCurrentProcess();
@@ -37,7 +44,7 @@ void AppShellTest::PreRunTestOnMainThread() {
   extension_system_ = static_cast<ShellExtensionSystem*>(
       ExtensionSystem::Get(browser_context_));
   extension_system_->FinishInitialization();
-  DCHECK(base::MessageLoopForUI::IsCurrent());
+  DCHECK(base::MessageLoopCurrentForUI::IsSet());
   base::RunLoop().RunUntilIdle();
 }
 
