@@ -10,13 +10,11 @@
 #include "base/logging.h"
 #include "base/strings/string_piece.h"
 #include "base/threading/thread_task_runner_handle.h"
-#include "jingle/glue/chrome_async_socket.h"
+#include "jingle/glue/network_service_async_socket.h"
 #include "jingle/glue/task_pump.h"
-#include "jingle/glue/xmpp_client_socket_factory.h"
 #include "jingle/notifier/base/weak_xmpp_client.h"
 #include "net/socket/client_socket_factory.h"
 #include "net/ssl/ssl_config_service.h"
-#include "net/url_request/url_request_context.h"
 #include "third_party/libjingle_xmpp/xmpp/xmppclientsettings.h"
 
 namespace notifier {
@@ -25,34 +23,33 @@ XmppConnection::Delegate::~Delegate() {}
 
 namespace {
 
-buzz::AsyncSocket* CreateSocket(
-    const buzz::XmppClientSettings& xmpp_client_settings,
-    const scoped_refptr<net::URLRequestContextGetter>& request_context_getter) {
+jingle_xmpp::AsyncSocket* CreateSocket(
+    const jingle_xmpp::XmppClientSettings& xmpp_client_settings,
+    jingle_glue::GetProxyResolvingSocketFactoryCallback
+        get_socket_factory_callback,
+    const net::NetworkTrafficAnnotationTag& traffic_annotation) {
   bool use_fake_ssl_client_socket =
-      (xmpp_client_settings.protocol() == cricket::PROTO_SSLTCP);
+      (xmpp_client_settings.protocol() == jingle_xmpp::PROTO_SSLTCP);
   // The default SSLConfig is good enough for us for now.
   const net::SSLConfig ssl_config;
   // These numbers were taken from similar numbers in
   // XmppSocketAdapter.
   const size_t kReadBufSize = 64U * 1024U;
   const size_t kWriteBufSize = 64U * 1024U;
-  jingle_glue::XmppClientSocketFactory* const client_socket_factory =
-      new jingle_glue::XmppClientSocketFactory(
-          net::ClientSocketFactory::GetDefaultFactory(),
-          ssl_config,
-          request_context_getter,
-          use_fake_ssl_client_socket);
-  return new jingle_glue::ChromeAsyncSocket(client_socket_factory,
-                                            kReadBufSize, kWriteBufSize);
+  return new jingle_glue::NetworkServiceAsyncSocket(
+      get_socket_factory_callback, use_fake_ssl_client_socket, kReadBufSize,
+      kWriteBufSize, traffic_annotation);
 }
 
 }  // namespace
 
 XmppConnection::XmppConnection(
-    const buzz::XmppClientSettings& xmpp_client_settings,
-    const scoped_refptr<net::URLRequestContextGetter>& request_context_getter,
+    const jingle_xmpp::XmppClientSettings& xmpp_client_settings,
+    jingle_glue::GetProxyResolvingSocketFactoryCallback
+        get_socket_factory_callback,
     Delegate* delegate,
-    buzz::PreXmppAuth* pre_xmpp_auth)
+    jingle_xmpp::PreXmppAuth* pre_xmpp_auth,
+    const net::NetworkTrafficAnnotationTag& traffic_annotation)
     : task_pump_(new jingle_glue::TaskPump()),
       on_connect_called_(false),
       delegate_(delegate) {
@@ -67,13 +64,13 @@ XmppConnection::XmppConnection(
   weak_xmpp_client->SignalLogOutput.connect(
       this, &XmppConnection::OnOutputLog);
   const char kLanguage[] = "en";
-  buzz::XmppReturnStatus connect_status =
-      weak_xmpp_client->Connect(xmpp_client_settings, kLanguage,
-                                CreateSocket(xmpp_client_settings,
-                                             request_context_getter),
-                                pre_xmpp_auth);
-  // buzz::XmppClient::Connect() should never fail.
-  DCHECK_EQ(connect_status, buzz::XMPP_RETURN_OK);
+  jingle_xmpp::XmppReturnStatus connect_status = weak_xmpp_client->Connect(
+      xmpp_client_settings, kLanguage,
+      CreateSocket(xmpp_client_settings, get_socket_factory_callback,
+                   traffic_annotation),
+      pre_xmpp_auth);
+  // jingle_xmpp::XmppClient::Connect() should never fail.
+  DCHECK_EQ(connect_status, jingle_xmpp::XMPP_RETURN_OK);
   weak_xmpp_client->Start();
   weak_xmpp_client_ = weak_xmpp_client->AsWeakPtr();
 }
@@ -87,10 +84,10 @@ XmppConnection::~XmppConnection() {
   // things happen when the stack pops back up to the XmppClient's
   // (which is deleted by |task_pump_|) function.
   base::ThreadTaskRunnerHandle::Get()->DeleteSoon(FROM_HERE,
-                                                  task_pump_.release());
+                                                  std::move(task_pump_));
 }
 
-void XmppConnection::OnStateChange(buzz::XmppEngine::State state) {
+void XmppConnection::OnStateChange(jingle_xmpp::XmppEngine::State state) {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
   VLOG(1) << "XmppClient state changed to " << state;
   if (!weak_xmpp_client_.get()) {
@@ -102,7 +99,7 @@ void XmppConnection::OnStateChange(buzz::XmppEngine::State state) {
     return;
   }
   switch (state) {
-    case buzz::XmppEngine::STATE_OPEN:
+    case jingle_xmpp::XmppEngine::STATE_OPEN:
       if (on_connect_called_) {
         LOG(DFATAL) << "State changed to STATE_OPEN more than once";
       } else {
@@ -110,11 +107,11 @@ void XmppConnection::OnStateChange(buzz::XmppEngine::State state) {
         on_connect_called_ = true;
       }
       break;
-    case buzz::XmppEngine::STATE_CLOSED: {
+    case jingle_xmpp::XmppEngine::STATE_CLOSED: {
       int subcode = 0;
-      buzz::XmppEngine::Error error =
+      jingle_xmpp::XmppEngine::Error error =
           weak_xmpp_client_->GetError(&subcode);
-      const buzz::XmlElement* stream_error =
+      const jingle_xmpp::XmlElement* stream_error =
           weak_xmpp_client_->GetStreamError();
       ClearClient();
       Delegate* delegate = delegate_;
