@@ -8,6 +8,7 @@
 #include <functiondiscoverykeys_devpkey.h>
 #include <objbase.h>
 #include <stddef.h>
+#include <bitset>
 
 #include "base/bind.h"
 #include "base/callback.h"
@@ -21,6 +22,7 @@
 #include "base/win/scoped_handle.h"
 #include "base/win/scoped_propvariant.h"
 #include "base/win/scoped_variant.h"
+#include "base/win/windows_version.h"
 #include "media/audio/audio_device_description.h"
 #include "media/base/media_switches.h"
 
@@ -47,6 +49,7 @@ enum class UmaLogStep {
   CREATE_CLIENT,
   GET_MIX_FORMAT,
   GET_DEVICE_PERIOD,
+  GET_SHARED_MODE_ENGINE_PERIOD,
 };
 
 using UMALogCallback = base::RepeatingCallback<void(UmaLogStep, HRESULT)>;
@@ -91,6 +94,9 @@ void LogUMAPreferredOutputParams(UmaLogStep step, HRESULT hr) {
           "GetPreferredOutputStreamParametersWin.GetDevicePeriodResult",
           hr);
       break;
+    case UmaLogStep::GET_SHARED_MODE_ENGINE_PERIOD:
+      // TODO(crbug.com/892044): add histogram logging.
+      break;
   }
 }
 
@@ -128,6 +134,15 @@ ChannelLayout ChannelConfigToChannelLayout(ChannelConfig config) {
     case KSAUDIO_SPEAKER_7POINT1_SURROUND:
       DVLOG(2) << "KSAUDIO_SPEAKER_7POINT1_SURROUND=>CHANNEL_LAYOUT_7_1";
       return CHANNEL_LAYOUT_7_1;
+    case KSAUDIO_SPEAKER_DIRECTOUT:
+      // When specifying the wave format for a direct-out stream, an application
+      // should set the dwChannelMask member of the WAVEFORMATEXTENSIBLE
+      // structure to the value KSAUDIO_SPEAKER_DIRECTOUT, which is zero.
+      // A channel mask of zero indicates that no speaker positions are defined.
+      // As always, the number of channels in the stream is specified in the
+      // Format.nChannels member.
+      DVLOG(2) << "KSAUDIO_SPEAKER_DIRECTOUT=>CHANNEL_LAYOUT_DISCRETE";
+      return CHANNEL_LAYOUT_DISCRETE;
     default:
       DVLOG(2) << "Unsupported channel configuration: " << config;
       return CHANNEL_LAYOUT_UNSUPPORTED;
@@ -137,6 +152,9 @@ ChannelLayout ChannelConfigToChannelLayout(ChannelConfig config) {
 // TODO(henrika): add mapping for all types in the ChannelLayout enumerator.
 ChannelConfig ChannelLayoutToChannelConfig(ChannelLayout layout) {
   switch (layout) {
+    case CHANNEL_LAYOUT_DISCRETE:
+      DVLOG(2) << "CHANNEL_LAYOUT_DISCRETE=>KSAUDIO_SPEAKER_DIRECTOUT";
+      return KSAUDIO_SPEAKER_DIRECTOUT;
     case CHANNEL_LAYOUT_MONO:
       DVLOG(2) << "CHANNEL_LAYOUT_MONO=>KSAUDIO_SPEAKER_MONO";
       return KSAUDIO_SPEAKER_MONO;
@@ -167,17 +185,122 @@ ChannelConfig ChannelLayoutToChannelConfig(ChannelLayout layout) {
   }
 }
 
-std::ostream& operator<<(std::ostream& os, const WAVEFORMATPCMEX& format) {
-  os << "wFormatTag: 0x" << std::hex << format.Format.wFormatTag
-     << ", nChannels: " << std::dec << format.Format.nChannels
-     << ", nSamplesPerSec: " << format.Format.nSamplesPerSec
-     << ", nAvgBytesPerSec: " << format.Format.nAvgBytesPerSec
-     << ", nBlockAlign: " << format.Format.nBlockAlign
-     << ", wBitsPerSample: " << format.Format.wBitsPerSample
-     << ", cbSize: " << format.Format.cbSize
-     << ", wValidBitsPerSample: " << format.Samples.wValidBitsPerSample
-     << ", dwChannelMask: 0x" << std::hex << format.dwChannelMask;
-  return os;
+// Converts the most common format tags defined in mmreg.h into string
+// equivalents. Mainly intended for log messages.
+const char* WaveFormatTagToString(WORD format_tag) {
+  switch (format_tag) {
+    case WAVE_FORMAT_UNKNOWN:
+      return "WAVE_FORMAT_UNKNOWN";
+    case WAVE_FORMAT_PCM:
+      return "WAVE_FORMAT_PCM";
+    case WAVE_FORMAT_IEEE_FLOAT:
+      return "WAVE_FORMAT_IEEE_FLOAT";
+    case WAVE_FORMAT_EXTENSIBLE:
+      return "WAVE_FORMAT_EXTENSIBLE";
+    default:
+      return "UNKNOWN";
+  }
+}
+
+// Converts from channel mask to list of included channels.
+// Each audio data format contains channels for one or more of the positions
+// listed below. The number of channels simply equals the number of nonzero
+// flag bits in the |channel_mask|. The relative positions of the channels
+// within each block of audio data always follow the same relative ordering
+// as the flag bits in the table below. For example, if |channel_mask| contains
+// the value 0x00000033, the format defines four audio channels that are
+// assigned for playback to the front-left, front-right, back-left,
+// and back-right speakers, respectively. The channel data should be interleaved
+// in that order within each block.
+std::string ChannelMaskToString(DWORD channel_mask) {
+  std::string ss;
+  if (channel_mask & SPEAKER_FRONT_LEFT)
+    ss += "FRONT_LEFT | ";
+  if (channel_mask & SPEAKER_FRONT_RIGHT)
+    ss += "FRONT_RIGHT | ";
+  if (channel_mask & SPEAKER_FRONT_CENTER)
+    ss += "FRONT_CENTER | ";
+  if (channel_mask & SPEAKER_LOW_FREQUENCY)
+    ss += "LOW_FREQUENCY | ";
+  if (channel_mask & SPEAKER_BACK_LEFT)
+    ss += "BACK_LEFT | ";
+  if (channel_mask & SPEAKER_BACK_RIGHT)
+    ss += "BACK_RIGHT | ";
+  if (channel_mask & SPEAKER_FRONT_LEFT_OF_CENTER)
+    ss += "FRONT_LEFT_OF_CENTER | ";
+  if (channel_mask & SPEAKER_FRONT_RIGHT_OF_CENTER)
+    ss += "RIGHT_OF_CENTER | ";
+  if (channel_mask & SPEAKER_BACK_CENTER)
+    ss += "BACK_CENTER | ";
+  if (channel_mask & SPEAKER_SIDE_LEFT)
+    ss += "SIDE_LEFT | ";
+  if (channel_mask & SPEAKER_SIDE_RIGHT)
+    ss += "SIDE_RIGHT | ";
+  if (channel_mask & SPEAKER_TOP_CENTER)
+    ss += "TOP_CENTER | ";
+  if (channel_mask & SPEAKER_TOP_FRONT_LEFT)
+    ss += "TOP_FRONT_LEFT | ";
+  if (channel_mask & SPEAKER_TOP_FRONT_CENTER)
+    ss += "TOP_FRONT_CENTER | ";
+  if (channel_mask & SPEAKER_TOP_FRONT_RIGHT)
+    ss += "TOP_FRONT_RIGHT | ";
+  if (channel_mask & SPEAKER_TOP_BACK_LEFT)
+    ss += "TOP_BACK_LEFT | ";
+  if (channel_mask & SPEAKER_TOP_BACK_CENTER)
+    ss += "TOP_BACK_CENTER | ";
+  if (channel_mask & SPEAKER_TOP_BACK_RIGHT)
+    ss += "TOP_BACK_RIGHT | ";
+
+  if (!ss.empty()) {
+    // Delete last appended " | " substring.
+    ss.erase(ss.end() - 3, ss.end());
+  }
+
+  std::bitset<8 * sizeof(DWORD)> mask(channel_mask);
+  ss += " (";
+  ss += std::to_string(mask.count());
+  ss += ")";
+  return ss;
+}
+
+// Converts a channel count into a channel configuration.
+ChannelConfig GuessChannelConfig(WORD channels) {
+  switch (channels) {
+    case 0:
+      DVLOG(2) << "KSAUDIO_SPEAKER_DIRECTOUT";
+      return KSAUDIO_SPEAKER_DIRECTOUT;
+    case 1:
+      DVLOG(2) << "KSAUDIO_SPEAKER_MONO";
+      return KSAUDIO_SPEAKER_MONO;
+    case 2:
+      DVLOG(2) << "KSAUDIO_SPEAKER_STEREO";
+      return KSAUDIO_SPEAKER_STEREO;
+    case 3:
+      DVLOG(2) << "KSAUDIO_SPEAKER_2POINT1";
+      return KSAUDIO_SPEAKER_2POINT1;
+    case 4:
+      DVLOG(2) << "KSAUDIO_SPEAKER_QUAD";
+      return KSAUDIO_SPEAKER_QUAD;
+    case 5:
+      DVLOG(2) << "KSAUDIO_SPEAKER_5POINT0";
+      return KSAUDIO_SPEAKER_5POINT0;
+    case 6:
+      DVLOG(2) << "KSAUDIO_SPEAKER_5POINT1";
+      return KSAUDIO_SPEAKER_5POINT1;
+    case 7:
+      DVLOG(2) << "KSAUDIO_SPEAKER_7POINT0";
+      return KSAUDIO_SPEAKER_7POINT0;
+    case 8:
+      DVLOG(2) << "KSAUDIO_SPEAKER_7POINT1";
+      return KSAUDIO_SPEAKER_7POINT1;
+    default:
+      DVLOG(1) << "Unsupported channel count: " << channels;
+  }
+  return KSAUDIO_SPEAKER_STEREO;
+}
+
+bool IAudioClient3IsSupported() {
+  return CoreAudioUtil::GetIAudioClientVersion() >= 3;
 }
 
 std::string GetDeviceID(IMMDevice* device) {
@@ -241,7 +364,15 @@ ComPtr<IMMDeviceEnumerator> CreateDeviceEnumeratorInternal(
   return device_enumerator;
 }
 
-ChannelLayout GetChannelLayout(const WAVEFORMATPCMEX& mix_format) {
+ChannelLayout GetChannelLayout(
+    const CoreAudioUtil::WaveFormatWrapper mix_format) {
+  if (!mix_format.IsExtensible()) {
+    DVLOG(1) << "Format does not contain any channel mask."
+             << " Guessing layout by channel count: " << std::dec
+             << mix_format->nChannels;
+    return GuessChannelLayout(mix_format->nChannels);
+  }
+
   // Get the integer mask which corresponds to the channel layout the
   // audio engine uses for its internal processing/mixing of shared-mode
   // streams. This mask indicates which channels are present in the multi-
@@ -251,9 +382,9 @@ ChannelLayout GetChannelLayout(const WAVEFORMATPCMEX& mix_format) {
   // See
   // http://msdn.microsoft.com/en-us/library/windows/hardware/ff537083.aspx
   // for more details.
-  ChannelConfig channel_config = mix_format.dwChannelMask;
+  ChannelConfig channel_config = mix_format.GetExtensible()->dwChannelMask;
 
-  // Convert Microsoft's channel configuration to genric ChannelLayout.
+  // Convert Microsoft's channel configuration to generic ChannelLayout.
   ChannelLayout channel_layout = ChannelConfigToChannelLayout(channel_config);
 
   // Some devices don't appear to set a valid channel layout, so guess based
@@ -261,9 +392,10 @@ ChannelLayout GetChannelLayout(const WAVEFORMATPCMEX& mix_format) {
   if (channel_layout == CHANNEL_LAYOUT_UNSUPPORTED) {
     DVLOG(1) << "Unsupported channel config: " << std::hex << channel_config
              << ".  Guessing layout by channel count: " << std::dec
-             << mix_format.Format.nChannels;
-    channel_layout = GuessChannelLayout(mix_format.Format.nChannels);
+             << mix_format->nChannels;
+    channel_layout = GuessChannelLayout(mix_format->nChannels);
   }
+  DVLOG(1) << "channel layout: " << ChannelLayoutToString(channel_layout);
 
   return channel_layout;
 }
@@ -363,38 +495,106 @@ ComPtr<IAudioClient> CreateClientInternal(IMMDevice* audio_device,
   return audio_client;
 }
 
+// Creates and activates an IAudioClient3 COM object given the selected
+// endpoint device.
+ComPtr<IAudioClient3> CreateClientInternal3(IMMDevice* audio_device,
+                                            const UMALogCallback& uma_log_cb) {
+  if (!audio_device)
+    return ComPtr<IAudioClient3>();
+
+  ComPtr<IAudioClient3> audio_client;
+  HRESULT hr = audio_device->Activate(
+      __uuidof(IAudioClient3), CLSCTX_INPROC_SERVER, NULL, &audio_client);
+  DVLOG_IF(1, FAILED(hr)) << "IMMDevice::Activate: " << std::hex << hr;
+  uma_log_cb.Run(UmaLogStep::CREATE_CLIENT, hr);
+  return audio_client;
+}
+
 HRESULT GetPreferredAudioParametersInternal(IAudioClient* client,
+                                            bool is_output_device,
                                             AudioParameters* params,
                                             const UMALogCallback& uma_log_cb) {
-  WAVEFORMATPCMEX mix_format;
+  WAVEFORMATEXTENSIBLE mix_format;
   HRESULT hr = CoreAudioUtil::GetSharedModeMixFormat(client, &mix_format);
   uma_log_cb.Run(UmaLogStep::GET_MIX_FORMAT, hr);
   if (FAILED(hr))
     return hr;
+  CoreAudioUtil::WaveFormatWrapper format(&mix_format);
 
-  REFERENCE_TIME default_period = 0;
-  hr = CoreAudioUtil::GetDevicePeriod(client, AUDCLNT_SHAREMODE_SHARED,
-                                      &default_period);
-  uma_log_cb.Run(UmaLogStep::GET_DEVICE_PERIOD, hr);
-  if (FAILED(hr))
-    return hr;
+  int min_frames_per_buffer = 0;
+  int max_frames_per_buffer = 0;
+  int frames_per_buffer = 0;
 
-  ChannelLayout channel_layout = GetChannelLayout(mix_format);
+  const bool supports_iac3 = IAudioClient3IsSupported();
+
+  if (supports_iac3) {
+    // Try to obtain an IAudioClient3 interface from the IAudioClient object.
+    // Use ComPtr::As for doing QueryInterface calls on COM objects.
+    ComPtr<IAudioClient> audio_client(client);
+    ComPtr<IAudioClient3> audio_client_3;
+    hr = audio_client.As(&audio_client_3);
+    if (SUCCEEDED(hr)) {
+      UINT32 default_period_frames = 0;
+      UINT32 fundamental_period_frames = 0;
+      UINT32 min_period_frames = 0;
+      UINT32 max_period_frames = 0;
+      hr = audio_client_3->GetSharedModeEnginePeriod(
+          format.get(), &default_period_frames, &fundamental_period_frames,
+          &min_period_frames, &max_period_frames);
+
+      uma_log_cb.Run(UmaLogStep::GET_SHARED_MODE_ENGINE_PERIOD, hr);
+      if (SUCCEEDED(hr)) {
+        min_frames_per_buffer = min_period_frames;
+        max_frames_per_buffer = max_period_frames;
+        frames_per_buffer = default_period_frames;
+      }
+      DVLOG(1) << "IAudioClient3 => min_period_frames: " << min_period_frames;
+      DVLOG(1) << "IAudioClient3 => frames_per_buffer: " << frames_per_buffer;
+    }
+  }
 
   // Preferred sample rate.
-  int sample_rate = mix_format.Format.nSamplesPerSec;
+  const int sample_rate = format->nSamplesPerSec;
 
-  // We are using the native device period to derive the smallest possible
-  // buffer size in shared mode. Note that the actual endpoint buffer will be
-  // larger than this size but it will be possible to fill it up in two calls.
-  // TODO(henrika): ensure that this scheme works for capturing as well.
-  int frames_per_buffer = static_cast<int>(
-      sample_rate *
-          CoreAudioUtil::ReferenceTimeToTimeDelta(default_period).InSecondsF() +
-      0.5);
+  // If we don't have access to IAudioClient3 or if the call to
+  // GetSharedModeEnginePeriod() fails we fall back to GetDevicePeriod().
+  if (!supports_iac3 || FAILED(hr)) {
+    REFERENCE_TIME default_period = 0;
+    hr = CoreAudioUtil::GetDevicePeriod(client, AUDCLNT_SHAREMODE_SHARED,
+                                        &default_period);
+    uma_log_cb.Run(UmaLogStep::GET_DEVICE_PERIOD, hr);
+    if (FAILED(hr))
+      return hr;
 
-  AudioParameters audio_params(AudioParameters::AUDIO_PCM_LOW_LATENCY,
-                               channel_layout, sample_rate, frames_per_buffer);
+    // We are using the native device period to derive the smallest possible
+    // buffer size in shared mode. Note that the actual endpoint buffer will be
+    // larger than this size but it will be possible to fill it up in two calls.
+    frames_per_buffer = static_cast<int>(
+        sample_rate * CoreAudioUtil::ReferenceTimeToTimeDelta(default_period)
+                          .InSecondsF() +
+        0.5);
+    DVLOG(1) << "IAudioClient => frames_per_buffer: " << frames_per_buffer;
+  }
+
+  ChannelLayout channel_layout = GetChannelLayout(format);
+  AudioParameters audio_params(
+      AudioParameters::AUDIO_PCM_LOW_LATENCY, channel_layout, sample_rate,
+      frames_per_buffer,
+      AudioParameters::HardwareCapabilities(min_frames_per_buffer,
+                                            max_frames_per_buffer));
+  // Set the number of channels explicitly to two for input devices if
+  // the channel layout is discrete to ensure that the parameters are valid
+  // and that clients does not have to support multi-channel input cases.
+  // Any required down-mixing from N (N > 2) to 2 must be performed by the
+  // input stream implementation instead.
+  // See https://crbug/868026 for examples where this approach is needed.
+  if (!is_output_device &&
+      audio_params.channel_layout() == CHANNEL_LAYOUT_DISCRETE) {
+    DLOG(WARNING)
+        << "Forcing number of channels to 2 for CHANNEL_LAYOUT_DISCRETE";
+    audio_params.set_channels_for_discrete(2);
+  }
+  DCHECK(audio_params.IsValid());
   *params = audio_params;
   DVLOG(1) << params->AsHumanReadableString();
 
@@ -403,16 +603,83 @@ HRESULT GetPreferredAudioParametersInternal(IAudioClient* client,
 
 }  // namespace
 
+// CoreAudioUtil::WaveFormatWrapper implementation.
+WAVEFORMATEXTENSIBLE* CoreAudioUtil::WaveFormatWrapper::GetExtensible() const {
+  CHECK(IsExtensible());
+  return reinterpret_cast<WAVEFORMATEXTENSIBLE*>(ptr_);
+}
+
+bool CoreAudioUtil::WaveFormatWrapper::IsExtensible() const {
+  return ptr_->wFormatTag == WAVE_FORMAT_EXTENSIBLE && ptr_->cbSize >= 22;
+}
+
+bool CoreAudioUtil::WaveFormatWrapper::IsPcm() const {
+  return IsExtensible() ? GetExtensible()->SubFormat == KSDATAFORMAT_SUBTYPE_PCM
+                        : ptr_->wFormatTag == WAVE_FORMAT_PCM;
+}
+
+bool CoreAudioUtil::WaveFormatWrapper::IsFloat() const {
+  return IsExtensible()
+             ? GetExtensible()->SubFormat == KSDATAFORMAT_SUBTYPE_IEEE_FLOAT
+             : ptr_->wFormatTag == WAVE_FORMAT_IEEE_FLOAT;
+}
+
+size_t CoreAudioUtil::WaveFormatWrapper::size() const {
+  return sizeof(*ptr_) + ptr_->cbSize;
+}
+
 bool CoreAudioUtil::IsSupported() {
   static bool g_is_supported = IsSupportedInternal();
   return g_is_supported;
 }
 
+// CoreAudioUtil implementation.
+std::string CoreAudioUtil::WaveFormatToString(const WaveFormatWrapper format) {
+  // Start with the WAVEFORMATEX part.
+  std::string wave_format = base::StringPrintf(
+      "wFormatTag: %s (0x%X), nChannels: %d, nSamplesPerSec: %lu"
+      ", nAvgBytesPerSec: %lu, nBlockAlign: %d, wBitsPerSample: %d, cbSize: %d",
+      WaveFormatTagToString(format->wFormatTag), format->wFormatTag,
+      format->nChannels, format->nSamplesPerSec, format->nAvgBytesPerSec,
+      format->nBlockAlign, format->wBitsPerSample, format->cbSize);
+  if (!format.IsExtensible())
+    return wave_format;
+
+  // Append the WAVEFORMATEXTENSIBLE part (which we know exists).
+  base::StringAppendF(
+      &wave_format, " [+] wValidBitsPerSample: %d, dwChannelMask: %s",
+      format.GetExtensible()->Samples.wValidBitsPerSample,
+      ChannelMaskToString(format.GetExtensible()->dwChannelMask).c_str());
+  if (format.IsPcm()) {
+    base::StringAppendF(&wave_format, "%s",
+                        ", SubFormat: KSDATAFORMAT_SUBTYPE_PCM");
+  } else if (format.IsFloat()) {
+    base::StringAppendF(&wave_format, "%s",
+                        ", SubFormat: KSDATAFORMAT_SUBTYPE_IEEE_FLOAT");
+  } else {
+    base::StringAppendF(&wave_format, "%s", ", SubFormat: NOT_SUPPORTED");
+  }
+  return wave_format;
+}
+
 base::TimeDelta CoreAudioUtil::ReferenceTimeToTimeDelta(REFERENCE_TIME time) {
   // Each unit of reference time is 100 nanoseconds <=> 0.1 microsecond.
   return base::TimeDelta::FromMicroseconds(0.1 * time + 0.5);
 }
 
+uint32_t CoreAudioUtil::GetIAudioClientVersion() {
+  if (base::win::GetVersion() >= base::win::VERSION_WIN10) {
+    // Minimum supported client: Windows 10.
+    // Minimum supported server: Windows Server 2016
+    return 3;
+  } else if (base::win::GetVersion() >= base::win::VERSION_WIN8) {
+    // Minimum supported client: Windows 8.
+    // Minimum supported server: Windows Server 2012.
+    return 2;
+  }
+  return 1;
+}
+
 AUDCLNT_SHAREMODE CoreAudioUtil::GetShareMode() {
   const base::CommandLine* cmd_line = base::CommandLine::ForCurrentProcess();
   if (cmd_line->HasSwitch(switches::kEnableExclusiveAudio))
@@ -631,44 +898,79 @@ ComPtr<IAudioClient> CoreAudioUtil::CreateClient(const std::string& device_id,
                                                  EDataFlow data_flow,
                                                  ERole role) {
   ComPtr<IMMDevice> device(CreateDevice(device_id, data_flow, role));
-
   return CreateClientInternal(device.Get(),
                               base::BindRepeating(&LogUMAEmptyCb));
 }
 
-HRESULT CoreAudioUtil::GetSharedModeMixFormat(
-    IAudioClient* client, WAVEFORMATPCMEX* format) {
-  ScopedCoMem<WAVEFORMATPCMEX> format_pcmex;
-  HRESULT hr = client->GetMixFormat(
-      reinterpret_cast<WAVEFORMATEX**>(&format_pcmex));
+ComPtr<IAudioClient3> CoreAudioUtil::CreateClient3(const std::string& device_id,
+                                                   EDataFlow data_flow,
+                                                   ERole role) {
+  ComPtr<IMMDevice> device(CreateDevice(device_id, data_flow, role));
+  return CreateClientInternal3(device.Get(),
+                               base::BindRepeating(&LogUMAEmptyCb));
+}
+
+HRESULT CoreAudioUtil::GetSharedModeMixFormat(IAudioClient* client,
+                                              WAVEFORMATEXTENSIBLE* format) {
+  // The GetMixFormat method retrieves the stream format that the audio engine
+  // uses for its internal processing of shared-mode streams. The method
+  // allocates the storage for the structure and this memory will be released
+  // when |mix_format| goes out of scope. The GetMixFormat method retrieves a
+  // format descriptor that is in the form of a WAVEFORMATEXTENSIBLE structure
+  // instead of a standalone WAVEFORMATEX structure. The method outputs a
+  // pointer to the WAVEFORMATEX structure that is embedded at the start of
+  // this WAVEFORMATEXTENSIBLE structure.
+  // Note that, crbug/803056 indicates that some devices can return a format
+  // where only the WAVEFORMATEX parts is initialized and we must be able to
+  // account for that.
+  ScopedCoMem<WAVEFORMATEXTENSIBLE> mix_format;
+  HRESULT hr =
+      client->GetMixFormat(reinterpret_cast<WAVEFORMATEX**>(&mix_format));
   if (FAILED(hr))
     return hr;
 
-  size_t bytes = sizeof(WAVEFORMATEX) + format_pcmex->Format.cbSize;
-  DCHECK_EQ(bytes, sizeof(WAVEFORMATPCMEX))
-      << "Format tag: 0x" << std::hex << format_pcmex->Format.wFormatTag;
+  // Use a wave format wrapper to make things simpler.
+  WaveFormatWrapper wrapped_format(mix_format.get());
 
-  memcpy(format, format_pcmex, bytes);
-  DVLOG(2) << *format;
+  // Verify that the reported format can be mixed by the audio engine in
+  // shared mode.
+  if (!wrapped_format.IsPcm() && !wrapped_format.IsFloat()) {
+    DLOG(ERROR)
+        << "Only pure PCM or float audio streams can be mixed in shared mode";
+    return AUDCLNT_E_UNSUPPORTED_FORMAT;
+  }
+  // Log a warning for the rare case where |mix_format| only contains a
+  // stand-alone WAVEFORMATEX structure but don't return.
+  if (!wrapped_format.IsExtensible()) {
+    DLOG(WARNING) << "The returned format contains no extended information. "
+                     "The size is "
+                  << wrapped_format.size() << " bytes.";
+  }
+
+  // Copy the correct number of bytes into |*format| taking into account if
+  // the returned structure is correctly extended or not.
+  CHECK_LE(wrapped_format.size(), sizeof(WAVEFORMATEXTENSIBLE))
+      << "Format tag: 0x" << std::hex << wrapped_format->wFormatTag;
+  memcpy(format, wrapped_format.get(), wrapped_format.size());
+  DVLOG(2) << CoreAudioUtil::WaveFormatToString(format);
 
   return hr;
 }
 
 bool CoreAudioUtil::IsFormatSupported(IAudioClient* client,
                                       AUDCLNT_SHAREMODE share_mode,
-                                      const WAVEFORMATPCMEX* format) {
-  ScopedCoMem<WAVEFORMATEXTENSIBLE> closest_match;
-  HRESULT hr = client->IsFormatSupported(
-      share_mode, reinterpret_cast<const WAVEFORMATEX*>(format),
-      reinterpret_cast<WAVEFORMATEX**>(&closest_match));
+                                      const WaveFormatWrapper format) {
+  ScopedCoMem<WAVEFORMATEX> closest_match;
+  HRESULT hr = client->IsFormatSupported(share_mode, format, &closest_match);
 
   // This log can only be triggered for shared mode.
   DLOG_IF(ERROR, hr == S_FALSE) << "Format is not supported "
                                 << "but a closest match exists.";
   // This log can be triggered both for shared and exclusive modes.
   DLOG_IF(ERROR, hr == AUDCLNT_E_UNSUPPORTED_FORMAT) << "Unsupported format.";
+  DVLOG(2) << CoreAudioUtil::WaveFormatToString(format);
   if (hr == S_FALSE) {
-    DVLOG(2) << *closest_match;
+    DVLOG(2) << CoreAudioUtil::WaveFormatToString(closest_match.get());
   }
 
   return (hr == S_OK);
@@ -683,31 +985,34 @@ bool CoreAudioUtil::IsChannelLayoutSupported(const std::string& device_id,
   if (!client.Get())
     return false;
 
-  WAVEFORMATPCMEX format;
-  HRESULT hr = GetSharedModeMixFormat(client.Get(), &format);
+  WAVEFORMATEXTENSIBLE mix_format;
+  HRESULT hr = GetSharedModeMixFormat(client.Get(), &mix_format);
   if (FAILED(hr))
     return false;
 
   // Next, check if it is possible to use an alternative format where the
   // channel layout (and possibly number of channels) is modified.
 
-  // Convert generic channel layout into Windows-specific channel configuration.
-  ChannelConfig new_config = ChannelLayoutToChannelConfig(channel_layout);
-  if (new_config == KSAUDIO_SPEAKER_UNSUPPORTED) {
-    return false;
+  // Convert generic channel layout into Windows-specific channel configuration
+  // but only if the wave format is extended (can contain a channel mask).
+  WaveFormatWrapper format(&mix_format);
+  if (format.IsExtensible()) {
+    ChannelConfig new_config = ChannelLayoutToChannelConfig(channel_layout);
+    if (new_config == KSAUDIO_SPEAKER_UNSUPPORTED) {
+      return false;
+    }
+    format.GetExtensible()->dwChannelMask = new_config;
   }
-  format.dwChannelMask = new_config;
 
   // Modify the format if the new channel layout has changed the number of
   // utilized channels.
   const int channels = ChannelLayoutToChannelCount(channel_layout);
-  if (channels != format.Format.nChannels) {
-    format.Format.nChannels = channels;
-    format.Format.nBlockAlign = (format.Format.wBitsPerSample / 8) * channels;
-    format.Format.nAvgBytesPerSec = format.Format.nSamplesPerSec *
-                                    format.Format.nBlockAlign;
+  if (channels != format->nChannels) {
+    format->nChannels = channels;
+    format->nBlockAlign = (format->wBitsPerSample / 8) * channels;
+    format->nAvgBytesPerSec = format->nSamplesPerSec * format->nBlockAlign;
   }
-  DVLOG(2) << format;
+  DVLOG(2) << CoreAudioUtil::WaveFormatToString(format);
 
   // Some devices can initialize a shared-mode stream with a format that is
   // not identical to the mix format obtained from the GetMixFormat() method.
@@ -718,7 +1023,7 @@ bool CoreAudioUtil::IsChannelLayoutSupported(const std::string& device_id,
   // for the audio device includes a local effects (LFX) audio processing
   // object (APO) that can handle format conversions.
   return CoreAudioUtil::IsFormatSupported(client.Get(),
-                                          AUDCLNT_SHAREMODE_SHARED, &format);
+                                          AUDCLNT_SHAREMODE_SHARED, format);
 }
 
 HRESULT CoreAudioUtil::GetDevicePeriod(IAudioClient* client,
@@ -731,8 +1036,8 @@ HRESULT CoreAudioUtil::GetDevicePeriod(IAudioClient* client,
   if (FAILED(hr))
     return hr;
 
-  *device_period = (share_mode == AUDCLNT_SHAREMODE_SHARED) ? default_period :
-      minimum_period;
+  *device_period = (share_mode == AUDCLNT_SHAREMODE_SHARED) ? default_period
+                                                            : minimum_period;
   DVLOG(2) << "device_period: "
            << ReferenceTimeToTimeDelta(*device_period).InMillisecondsF()
            << " [ms]";
@@ -754,17 +1059,21 @@ HRESULT CoreAudioUtil::GetPreferredAudioParameters(const std::string& device_id,
   if (!client.Get())
     return E_FAIL;
 
-  HRESULT hr =
-      GetPreferredAudioParametersInternal(client.Get(), params, uma_log_cb);
-  if (FAILED(hr) || is_output_device || !params->IsValid())
+  HRESULT hr = GetPreferredAudioParametersInternal(
+      client.Get(), is_output_device, params, uma_log_cb);
+  if (FAILED(hr) || is_output_device || !params->IsValid()) {
     return hr;
+  }
 
   // The following functionality is only for input devices.
   DCHECK(!is_output_device);
 
   // TODO(dalecurtis): Old code rewrote != 1 channels to stereo, do we still
   // need to do the same thing?
-  if (params->channels() != 1) {
+  if (params->channels() != 1 &&
+      params->channel_layout() != CHANNEL_LAYOUT_DISCRETE) {
+    DLOG(WARNING)
+        << "Replacing existing audio parameter with predefined version";
     params->Reset(params->format(), CHANNEL_LAYOUT_STEREO,
                   params->sample_rate(), params->frames_per_buffer());
   }
@@ -776,16 +1085,27 @@ ChannelConfig CoreAudioUtil::GetChannelConfig(const std::string& device_id,
                                               EDataFlow data_flow) {
   ComPtr<IAudioClient> client(CreateClient(device_id, data_flow, eConsole));
 
-  WAVEFORMATPCMEX format = {};
-  if (!client.Get() || FAILED(GetSharedModeMixFormat(client.Get(), &format)))
+  WAVEFORMATEXTENSIBLE mix_format;
+  if (!client.Get() ||
+      FAILED(GetSharedModeMixFormat(client.Get(), &mix_format)))
     return 0;
+  WaveFormatWrapper format(&mix_format);
+  if (!format.IsExtensible()) {
+    // A format descriptor from WAVEFORMATEX only supports mono and stereo.
+    DCHECK_LE(format->nChannels, 2);
+    DVLOG(1) << "Format does not contain any channel mask."
+             << " Guessing layout by channel count: " << std::dec
+             << format->nChannels;
+    return GuessChannelConfig(format->nChannels);
+  }
 
-  return static_cast<ChannelConfig>(format.dwChannelMask);
+  return static_cast<ChannelConfig>(format.GetExtensible()->dwChannelMask);
 }
 
 HRESULT CoreAudioUtil::SharedModeInitialize(IAudioClient* client,
-                                            const WAVEFORMATPCMEX* format,
+                                            const WaveFormatWrapper format,
                                             HANDLE event_handle,
+                                            uint32_t requested_buffer_size,
                                             uint32_t* endpoint_buffer_size,
                                             const GUID* session_guid) {
   // Use default flags (i.e, dont set AUDCLNT_STREAMFLAGS_NOPERSIST) to
@@ -798,23 +1118,44 @@ HRESULT CoreAudioUtil::SharedModeInitialize(IAudioClient* client,
   // After the stream starts, the audio engine will signal the event handle
   // to notify the client each time a buffer becomes ready to process.
   // Event-driven buffering is supported for both rendering and capturing.
-  // Both shared-mode and exclusive-mode streams can use event-driven buffering.
-  bool use_event = (event_handle != NULL &&
-                    event_handle != INVALID_HANDLE_VALUE);
+  // Both shared-mode and exclusive-mode streams can use event-driven
+  // buffering.
+  bool use_event =
+      (event_handle != NULL && event_handle != INVALID_HANDLE_VALUE);
   if (use_event)
     stream_flags |= AUDCLNT_STREAMFLAGS_EVENTCALLBACK;
   DVLOG(2) << "stream_flags: 0x" << std::hex << stream_flags;
 
-  // Initialize the shared mode client for minimal delay.
-  HRESULT hr = client->Initialize(AUDCLNT_SHAREMODE_SHARED,
-                                  stream_flags,
-                                  0,
-                                  0,
-                                  reinterpret_cast<const WAVEFORMATEX*>(format),
-                                  session_guid);
-  if (FAILED(hr)) {
-    DVLOG(1) << "IAudioClient::Initialize: " << std::hex << hr;
-    return hr;
+  const bool supports_iac3 = IAudioClient3IsSupported();
+
+  HRESULT hr;
+  if (supports_iac3 && requested_buffer_size > 0) {
+    // Try to obtain an IAudioClient3 interface from the IAudioClient object.
+    // Use ComPtr::As for doing QueryInterface calls on COM objects.
+    ComPtr<IAudioClient> audio_client(client);
+    ComPtr<IAudioClient3> audio_client_3;
+    hr = audio_client.As(&audio_client_3);
+    if (FAILED(hr)) {
+      DVLOG(1) << "Failed to obtain IAudioClient3 interface: " << std::hex
+               << hr;
+      return hr;
+    }
+    // Initialize a low-latency client using IAudioClient3.
+    hr = audio_client_3->InitializeSharedAudioStream(
+        stream_flags, requested_buffer_size, format, session_guid);
+    if (FAILED(hr)) {
+      DVLOG(1) << "IAudioClient3::InitializeSharedAudioStream: " << std::hex
+               << hr;
+      return hr;
+    }
+  } else {
+    // Initialize the shared mode client for minimal delay.
+    hr = client->Initialize(AUDCLNT_SHAREMODE_SHARED, stream_flags, 0, 0,
+                            format, session_guid);
+    if (FAILED(hr)) {
+      DVLOG(1) << "IAudioClient::Initialize: " << std::hex << hr;
+      return hr;
+    }
   }
 
   if (use_event) {
@@ -836,7 +1177,7 @@ HRESULT CoreAudioUtil::SharedModeInitialize(IAudioClient* client,
   DVLOG(2) << "endpoint buffer size: " << buffer_size_in_frames;
 
   // TODO(henrika): utilize when delay measurements are added.
-  REFERENCE_TIME  latency = 0;
+  REFERENCE_TIME latency = 0;
   hr = client->GetStreamLatency(&latency);
   DVLOG(2) << "stream latency: "
            << ReferenceTimeToTimeDelta(latency).InMillisecondsF() << " [ms]";
