@@ -80,11 +80,11 @@ void ResourceMultiBufferDataProvider::Start() {
   }
 
   // Prepare the request.
-  auto request = std::make_unique<WebURLRequest>(url_data_->url());
-  request->SetRequestContext(is_client_audio_element_
-                                 ? blink::mojom::RequestContextType::AUDIO
-                                 : blink::mojom::RequestContextType::VIDEO);
-  request->SetHTTPHeaderField(
+  WebURLRequest request(url_data_->url());
+  request.SetRequestContext(is_client_audio_element_
+                                ? blink::mojom::RequestContextType::AUDIO
+                                : blink::mojom::RequestContextType::VIDEO);
+  request.SetHTTPHeaderField(
       WebString::FromUTF8(net::HttpRequestHeaders::kRange),
       WebString::FromUTF8(
           net::HttpByteRange::RightUnbounded(byte_pos()).GetHeaderValue()));
@@ -94,8 +94,8 @@ void ResourceMultiBufferDataProvider::Start() {
     // This lets the data reduction proxy know that we don't have anything
     // previously cached data for this resource. We can only send it if this is
     // the first request for this resource.
-    request->SetHTTPHeaderField(WebString::FromUTF8("chrome-proxy"),
-                                WebString::FromUTF8("frfr"));
+    request.SetHTTPHeaderField(WebString::FromUTF8("chrome-proxy"),
+                               WebString::FromUTF8("frfr"));
   }
 
   // We would like to send an if-match header with the request to
@@ -105,7 +105,7 @@ void ResourceMultiBufferDataProvider::Start() {
   // along the way. See crbug/504194 and crbug/689989 for more information.
 
   // Disable compression, compression for audio/video doesn't make sense...
-  request->SetHTTPHeaderField(
+  request.SetHTTPHeaderField(
       WebString::FromUTF8(net::HttpRequestHeaders::kAcceptEncoding),
       WebString::FromUTF8("identity;q=1, *;q=0"));
 
@@ -115,26 +115,18 @@ void ResourceMultiBufferDataProvider::Start() {
     options.expose_all_response_headers = true;
     // The author header set is empty, no preflight should go ahead.
     options.preflight_policy =
-        network::mojom::CORSPreflightPolicy::kPreventPreflight;
+        network::mojom::CorsPreflightPolicy::kPreventPreflight;
 
-    request->SetFetchRequestMode(network::mojom::FetchRequestMode::kCORS);
+    request.SetFetchRequestMode(network::mojom::FetchRequestMode::kCors);
     if (url_data_->cors_mode() != UrlData::CORS_USE_CREDENTIALS) {
-      request->SetFetchCredentialsMode(
+      request.SetFetchCredentialsMode(
           network::mojom::FetchCredentialsMode::kSameOrigin);
     }
   }
 
   active_loader_ =
       url_data_->url_index()->fetch_context()->CreateUrlLoader(options);
-
-  url_data_->WaitToLoad(
-      base::BindOnce(&ResourceMultiBufferDataProvider::StartLoading,
-                     weak_factory_.GetWeakPtr(), std::move(request)));
-}
-
-void ResourceMultiBufferDataProvider::StartLoading(
-    std::unique_ptr<WebURLRequest> request) {
-  active_loader_->LoadAsynchronously(*request, this);
+  active_loader_->LoadAsynchronously(request, this);
 }
 
 /////////////////////////////////////////////////////////////////////////////
@@ -337,7 +329,7 @@ void ResourceMultiBufferDataProvider::DidReceiveResponse(
 
   // This is vital for security!
   destination_url_data->set_is_cors_cross_origin(
-      network::cors::IsCORSCrossOriginResponseType(response_type));
+      network::cors::IsCorsCrossOriginResponseType(response_type));
 
   if (destination_url_data != url_data_) {
     // At this point, we've encountered a redirect, or found a better url data
@@ -365,11 +357,19 @@ void ResourceMultiBufferDataProvider::DidReceiveResponse(
     return;  // "this" may be deleted now.
   }
 
+  // Get the response URL since it can differ from the request URL when a
+  // service worker provided the response. Normally we would just use
+  // ResponseUrl(), but ResourceMultibufferDataProvider disallows mixing
+  // constructed responses (new Response()) and native server responses, even if
+  // they have the same response URL.
+  GURL response_url;
+  if (!response.WasFetchedViaServiceWorker() ||
+      response.HasUrlListViaServiceWorker()) {
+    response_url = response.ResponseUrl();
+  }
+
   // This test is vital for security!
-  const GURL& original_url = response.WasFetchedViaServiceWorker()
-                                 ? response.OriginalURLViaServiceWorker()
-                                 : response.Url();
-  if (!url_data_->ValidateDataOrigin(original_url.GetOrigin())) {
+  if (!url_data_->ValidateDataOrigin(response_url.GetOrigin())) {
     active_loader_.reset();
     url_data_->Fail();
     return;  // "this" may be deleted now.
@@ -421,7 +421,8 @@ void ResourceMultiBufferDataProvider::DidReceiveData(const char* data,
   // Beware, this object might be deleted here.
 }
 
-void ResourceMultiBufferDataProvider::DidDownloadData(int dataLength) {
+void ResourceMultiBufferDataProvider::DidDownloadData(
+    unsigned long long dataLength) {
   NOTIMPLEMENTED();
 }
 
