@@ -60,14 +60,15 @@ std::string MediaCodecAudioDecoder::GetDisplayName() const {
   return "MediaCodecAudioDecoder";
 }
 
-void MediaCodecAudioDecoder::Initialize(
-    const AudioDecoderConfig& config,
-    CdmContext* cdm_context,
-    const InitCB& init_cb,
-    const OutputCB& output_cb,
-    const WaitingForDecryptionKeyCB& /* waiting_for_decryption_key_cb */) {
+void MediaCodecAudioDecoder::Initialize(const AudioDecoderConfig& config,
+                                        CdmContext* cdm_context,
+                                        const InitCB& init_cb,
+                                        const OutputCB& output_cb,
+                                        const WaitingCB& waiting_cb) {
   DVLOG(1) << __func__ << ": " << config.AsHumanReadableString();
   DCHECK_NE(state_, STATE_WAITING_FOR_MEDIA_CRYPTO);
+  DCHECK(output_cb);
+  DCHECK(waiting_cb);
 
   // Initialization and reinitialization should not be called during pending
   // decode.
@@ -104,7 +105,11 @@ void MediaCodecAudioDecoder::Initialize(
   }
 
   config_ = config;
+
+  // TODO(xhwang): Check whether BindToCurrentLoop is needed here.
   output_cb_ = BindToCurrentLoop(output_cb);
+  waiting_cb_ = BindToCurrentLoop(waiting_cb);
+
   SetInitialConfiguration();
 
   if (config_.is_encrypted() && !media_crypto_) {
@@ -466,6 +471,11 @@ bool MediaCodecAudioDecoder::OnDecodedFrame(
   return true;
 }
 
+void MediaCodecAudioDecoder::OnWaiting(WaitingReason reason) {
+  DVLOG(2) << __func__;
+  waiting_cb_.Run(reason);
+}
+
 bool MediaCodecAudioDecoder::OnOutputFormatChanged() {
   DVLOG(2) << __func__;
   MediaCodecBridge* media_codec = codec_loop_->GetCodec();
