@@ -3,8 +3,8 @@
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 
+import argparse
 import multiprocessing
-import optparse
 import os
 import posixpath
 import sys
@@ -30,8 +30,7 @@ LIB_DICT = {
   'win': ['x86_32']
 }
 VALID_TOOLCHAINS = [
-  'bionic',
-  'newlib',
+  'clang-newlib',
   'glibc',
   'pnacl',
   'win',
@@ -99,6 +98,7 @@ def ValidateToolchains(toolchains):
     buildbot_common.ErrorExit('Invalid toolchain(s): %s' % (
         ', '.join(invalid_toolchains)))
 
+
 def GetDeps(projects):
   out = {}
 
@@ -147,7 +147,7 @@ def UpdateProjects(pepperdir, project_tree, toolchains,
     if clobber:
       buildbot_common.RemoveDir(dirpath)
     buildbot_common.MakeDir(dirpath)
-    targets = [desc['NAME'] for desc in projects]
+    targets = [desc['NAME'] for desc in projects if 'TARGETS' in desc]
     deps = GetDeps(projects)
 
     # Generate master make for this branch of projects
@@ -211,10 +211,6 @@ def BuildProjectsBranch(pepperdir, branch, deps, clean, config, args=None):
   make_cmd = [make, '-j', jobs]
 
   make_cmd.append('CONFIG='+config)
-  # We always ENABLE_BIONIC in case we need it.  If neither --bionic nor
-  # -t bionic have been provided on the command line, then VALID_TOOLCHAINS
-  # will not contain a bionic target.
-  make_cmd.append('ENABLE_BIONIC=1')
   if not deps:
     make_cmd.append('IGNORE_DEPS=1')
 
@@ -243,28 +239,28 @@ def BuildProjects(pepperdir, project_tree, deps=True,
     BuildProjectsBranch(pepperdir, branch, deps, clean, config)
 
 
-def main(argv):
-  parser = optparse.OptionParser()
-  parser.add_option('-c', '--clobber',
+def main(args):
+  parser = argparse.ArgumentParser(description=__doc__)
+  parser.add_argument('-c', '--clobber',
       help='Clobber project directories before copying new files',
       action='store_true', default=False)
-  parser.add_option('-b', '--build',
+  parser.add_argument('-b', '--build',
       help='Build the projects. Otherwise the projects are only copied.',
       action='store_true')
-  parser.add_option('--config',
+  parser.add_argument('--config',
       help='Choose configuration to build (Debug or Release).  Builds both '
            'by default')
-  parser.add_option('--bionic',
-      help='Enable bionic projects', action='store_true')
-  parser.add_option('-x', '--experimental',
+  parser.add_argument('-x', '--experimental',
       help='Build experimental projects', action='store_true')
-  parser.add_option('-t', '--toolchain',
+  parser.add_argument('-t', '--toolchain',
       help='Build using toolchain. Can be passed more than once.',
       action='append', default=[])
-  parser.add_option('-d', '--dest',
+  parser.add_argument('-d', '--dest',
       help='Select which build destinations (project types) are valid.',
       action='append')
-  parser.add_option('-v', '--verbose', action='store_true')
+  parser.add_argument('projects', nargs='*',
+      help='Select which projects to build.')
+  parser.add_argument('-v', '--verbose', action='store_true')
 
   # To setup bash completion for this command first install optcomplete
   # and then add this line to your .bashrc:
@@ -275,7 +271,7 @@ def main(argv):
   except ImportError:
     pass
 
-  options, args = parser.parse_args(argv[1:])
+  options = parser.parse_args(args)
 
   global verbose
   if options.verbose:
@@ -294,11 +290,9 @@ def main(argv):
   if not options.toolchain:
     # Order matters here: the default toolchain for an example's Makefile will
     # be the first toolchain in this list that is available in the example.
-    # e.g. If an example supports newlib and glibc, then the default will be
-    # newlib.
-    options.toolchain = ['pnacl', 'newlib', 'glibc', 'host']
-    if options.experimental or options.bionic:
-      options.toolchain.append('bionic')
+    # e.g. If an example supports clang-newlib and glibc, then the default will
+    # be clang-newlib.
+    options.toolchain = ['pnacl', 'clang-newlib', 'glibc', 'host']
 
   if 'host' in options.toolchain:
     options.toolchain.remove('host')
@@ -316,9 +310,9 @@ def main(argv):
   if options.dest:
     filters['DEST'] = options.dest
     Trace('Filter by type: ' + str(options.dest))
-  if args:
-    filters['NAME'] = args
-    Trace('Filter by name: ' + str(args))
+  if options.projects:
+    filters['NAME'] = options.projects
+    Trace('Filter by name: ' + str(options.projects))
 
   try:
     project_tree = parse_dsc.LoadProjectTree(SDK_SRC_DIR, include=filters)
@@ -346,7 +340,7 @@ def main(argv):
 if __name__ == '__main__':
   script_name = os.path.basename(sys.argv[0])
   try:
-    sys.exit(main(sys.argv))
+    sys.exit(main(sys.argv[1:]))
   except parse_dsc.ValidationError as e:
     buildbot_common.ErrorExit('%s: %s' % (script_name, e))
   except KeyboardInterrupt:
