@@ -8,7 +8,7 @@
 This script is normally run immediately after build_sdk.py.
 """
 
-import optparse
+import argparse
 import os
 import subprocess
 import sys
@@ -56,11 +56,31 @@ def StepCopyTests(pepperdir, toolchains, build_experimental):
                                 toolchains=toolchains)
 
 
-def StepBuildTests(pepperdir):
+def StepBuildLibraries(pepperdir, sanitizer):
   for config in ('Debug', 'Release'):
-    build_sdk.BuildStepMakeAll(pepperdir, 'tests',
-                                   'Build Tests (%s)' % config,
-                                   deps=False, config=config)
+    title = 'Build Libs (%s)[sanitizer=%s]' % (config, sanitizer)
+    build_sdk.BuildStepMakeAll(pepperdir, 'src', title, config=config,
+        args=GetSanitizerArgs(sanitizer))
+
+
+def StepBuildTests(pepperdir, sanitizer):
+  for config in ('Debug', 'Release'):
+    title = 'Build Tests (%s)' % config
+    if sanitizer:
+      title += '[sanitizer=%s]'  % sanitizer
+
+    build_sdk.BuildStepMakeAll(pepperdir, 'tests', title, deps=False,
+        config=config, args=GetSanitizerArgs(sanitizer))
+
+
+def GetSanitizerArgs(sanitizer):
+  if sanitizer == 'valgrind':
+    return ['TOOLCHAIN=linux', 'RUN_UNDER=valgrind']
+  elif sanitizer == 'address':
+    return ['TOOLCHAIN=linux', 'ASAN=1']
+  elif sanitizer == 'thread':
+    return ['TOOLCHAIN=linux', 'TSAN=1']
+  return []
 
 
 def StepRunSelLdrTests(pepperdir, sanitizer):
@@ -72,42 +92,23 @@ def StepRunSelLdrTests(pepperdir, sanitizer):
 
   def RunTest(test, toolchain, config, arch=None):
     args = ['STANDALONE=1', 'TOOLCHAIN=%s' % toolchain]
+    args += GetSanitizerArgs(sanitizer)
     if arch is not None:
       args.append('NACL_ARCH=%s' % arch)
-    deps = False
-
-    if sanitizer is not None:
-      # For sanitizer builds we pass extra argument for make, and do
-      # full clean build to make sure everything is rebuilt with the
-      # correct flags
-      deps = True
-      if sanitizer == 'valgrind':
-        args += ['RUN_UNDER=valgrind']
-      else:
-        args += ['CC=clang', 'CXX=clang++',
-                 'LDFLAGS=-pie -fsanitize=' + sanitizer,
-                 'CFLAGS=-fPIC -fsanitize=' + sanitizer]
-      build_projects.BuildProjectsBranch(pepperdir, 'src', clean=False,
-                                         deps=deps, config=config,
-                                         args=args + ['clean'])
-      build_projects.BuildProjectsBranch(pepperdir, 'tests', clean=False,
-                                         deps=deps, config=config,
-                                         args=args + ['clean'])
 
     build_projects.BuildProjectsBranch(pepperdir, test, clean=False,
-                                       deps=deps, config=config,
+                                       deps=False, config=config,
                                        args=args + ['run'])
 
   if getos.GetPlatform() == 'win':
-    # On win32 we only support running on the system
-    # arch
+    # On win32 we only support running on the system arch
     archs = (getos.GetSystemArch('win'),)
   elif getos.GetPlatform() == 'mac':
-    # We only ship 32-bit version of sel_ldr on mac.
-    archs = ('x86_32',)
-  else:
-    # On linux we can run both 32 and 64-bit
+    # On mac we can run both 32 and 64-bit
     archs = ('x86_64', 'x86_32')
+  else:
+    # On linux we can run both 32 and 64-bit, and arm (via qemu)
+    archs = ('x86_64', 'x86_32', 'arm')
 
   for root, projects in tree.iteritems():
     for project in projects:
@@ -132,7 +133,7 @@ def StepRunSelLdrTests(pepperdir, sanitizer):
       if sanitizer:
         continue
 
-      for toolchain in ('newlib', 'glibc'):
+      for toolchain in ('clang-newlib', 'glibc', 'pnacl'):
         for arch in archs:
           for config in configs:
             RunTest(location, toolchain, config, arch)
@@ -150,6 +151,10 @@ def StepRunBrowserTests(toolchains, experimental):
   if experimental:
     args.append('-x')
   for toolchain in toolchains:
+    # TODO(sbc): Re-enable brower tests for native PPAPI platforms:
+    # http://crbug.com/646666
+    if toolchain in ['mac', 'linux', 'win']:
+      continue
     args.extend(['-t', toolchain])
 
   try:
@@ -159,15 +164,15 @@ def StepRunBrowserTests(toolchains, experimental):
 
 
 def main(args):
-  usage = '%prog [<options>] [<phase...>]'
-  parser = optparse.OptionParser(description=__doc__, usage=usage)
-  parser.add_option('--experimental', help='build experimental tests',
-                    action='store_true')
-  parser.add_option('--sanitizer',
-                    help='Run sanitizer (asan/tsan/valgrind) tests',
-                    action='store_true')
-  parser.add_option('--verbose', '-v', help='Verbose output',
-                    action='store_true')
+  parser = argparse.ArgumentParser(description=__doc__)
+  parser.add_argument('--experimental', help='build experimental tests',
+                      action='store_true')
+  parser.add_argument('--sanitizer',
+                      help='Run sanitizer (asan/tsan/valgrind) tests',
+                      action='store_true')
+  parser.add_argument('--verbose', '-v', help='Verbose output',
+                      action='store_true')
+  parser.add_argument('phases', nargs="*")
 
   if 'NACL_SDK_ROOT' in os.environ:
     # We don't want the currently configured NACL_SDK_ROOT to have any effect
@@ -183,11 +188,11 @@ def main(args):
   except ImportError:
     pass
 
-  options, args = parser.parse_args(args[1:])
+  options = parser.parse_args(args)
 
   pepper_ver = str(int(build_version.ChromeMajorVersion()))
   pepperdir = os.path.join(OUT_DIR, 'pepper_' + pepper_ver)
-  toolchains = ['newlib', 'glibc', 'pnacl']
+  toolchains = ['clang-newlib', 'glibc', 'pnacl']
   toolchains.append(getos.GetPlatform())
 
   if options.verbose:
@@ -196,23 +201,36 @@ def main(args):
   phases = [
     ('build_examples', StepBuildExamples, pepperdir),
     ('copy_tests', StepCopyTests, pepperdir, toolchains, options.experimental),
-    ('build_tests', StepBuildTests, pepperdir),
-    ('sel_ldr_tests', StepRunSelLdrTests, pepperdir, None),
-    ('browser_tests', StepRunBrowserTests, toolchains, options.experimental),
+    ('build_tests', StepBuildTests, pepperdir, None),
   ]
 
   if options.sanitizer:
     if getos.GetPlatform() != 'linux':
       buildbot_common.ErrorExit('sanitizer tests only run on linux.')
+    clang_dir = os.path.join(SRC_DIR, 'third_party', 'llvm-build',
+        'Release+Asserts', 'bin')
+    os.environ['PATH'] = clang_dir + os.pathsep + os.environ['PATH']
+
     phases += [
+      ('build_libs_asan', StepBuildLibraries, pepperdir, 'address'),
+      ('build_libs_tsan', StepBuildLibraries, pepperdir, 'thread'),
+      ('build_tests_asan', StepBuildTests, pepperdir, 'address'),
+      ('build_tests_tsan', StepBuildTests, pepperdir, 'thread'),
       ('sel_ldr_tests_asan', StepRunSelLdrTests, pepperdir, 'address'),
       ('sel_ldr_tests_tsan', StepRunSelLdrTests, pepperdir, 'thread'),
-      ('sel_ldr_tests_valgrind', StepRunSelLdrTests, pepperdir, 'valgrind')
+      # TODO(sbc): get valgrind installed on the bots to enable this
+      # configuration
+      #('sel_ldr_tests_valgrind', StepRunSelLdrTests, pepperdir, 'valgrind')
+    ]
+  else:
+    phases += [
+      ('sel_ldr_tests', StepRunSelLdrTests, pepperdir, None),
+      ('browser_tests', StepRunBrowserTests, toolchains, options.experimental),
     ]
 
-  if args:
+  if options.phases:
     phase_names = [p[0] for p in phases]
-    for arg in args:
+    for arg in options.phases:
       if arg not in phase_names:
         msg = 'Invalid argument: %s\n' % arg
         msg += 'Possible arguments:\n'
@@ -222,7 +240,7 @@ def main(args):
 
   for phase in phases:
     phase_name = phase[0]
-    if args and phase_name not in args:
+    if options.phases and phase_name not in options.phases:
       continue
     phase_func = phase[1]
     phase_args = phase[2:]
@@ -233,6 +251,6 @@ def main(args):
 
 if __name__ == '__main__':
   try:
-    sys.exit(main(sys.argv))
+    sys.exit(main(sys.argv[1:]))
   except KeyboardInterrupt:
     buildbot_common.ErrorExit('test_sdk: interrupted')
