@@ -12,6 +12,7 @@
 #include <ppapi/c/pp_errors.h>
 
 #include "nacl_io/httpfs/http_fs.h"
+#include "nacl_io/http_status_codes.h"
 #include "nacl_io/kernel_handle.h"
 #include "nacl_io/osinttypes.h"
 
@@ -27,11 +28,6 @@ namespace {
 // request, we need to read all of the data up to the start of our partial
 // request into a dummy buffer. This is the maximum size of that buffer.
 const int MAX_READ_BUFFER_SIZE = 64 * 1024;
-const int32_t STATUSCODE_OK = 200;
-const int32_t STATUSCODE_PARTIAL_CONTENT = 206;
-const int32_t STATUSCODE_FORBIDDEN = 403;
-const int32_t STATUSCODE_NOT_FOUND = 404;
-const int32_t STATUSCODE_REQUESTED_RANGE_NOT_SATISFIABLE = 416;
 
 StringMap_t ParseHeaders(const char* headers, int32_t headers_length) {
   enum State {
@@ -66,8 +62,14 @@ StringMap_t ParseHeaders(const char* headers, int32_t headers_length) {
         // Found a non-whitespace, mark this as the start of the value.
         start = &headers[i];
         state = FINDING_VALUE;
-      // Fallthrough to start processing value without incrementing i.
 
+        // NOTE: Avoid fallthrough as it produces a warning on newer compilers,
+        // but can't easily be silenced by the older NaCl compilers.
+        //
+        // Fallthrough to start processing value without incrementing i.
+        goto finding_value;
+
+      finding_value:
       case FINDING_VALUE:
         if (headers[i] == '\n') {
           // Found value.
@@ -115,14 +117,20 @@ bool ParseContentRange(const StringMap_t& headers,
   // bytes is "bytes 0-9/*". Convert it to a half-open range by incrementing
   // read_end.
   if (result == 2) {
-    *read_start = read_start_int;
-    *read_end = read_end_int + 1;
-    *entity_length = 0;
+    if (read_start)
+      *read_start = read_start_int;
+    if (read_end)
+      *read_end = read_end_int + 1;
+    if (entity_length)
+      *entity_length = 0;
     return true;
   } else if (result == 3) {
-    *read_start = read_start_int;
-    *read_end = read_end_int + 1;
-    *entity_length = entity_length_int;
+    if (read_start)
+      *read_start = read_start_int;
+    if (read_end)
+      *read_end = read_end_int + 1;
+    if (entity_length)
+      *entity_length = entity_length_int;
     return true;
   }
 
@@ -222,12 +230,16 @@ HttpFsNode::HttpFsNode(Filesystem* filesystem,
                        bool cache_content)
     : Node(filesystem),
       url_(url),
+      buffer_(NULL),
+      buffer_len_(0),
       cache_content_(cache_content),
       has_cached_size_(false) {
+  // http nodes are read-only by default
+  SetMode(S_IRALL);
 }
 
-void HttpFsNode::SetMode(int mode) {
-  stat_.st_mode = mode;
+HttpFsNode::~HttpFsNode() {
+  free(buffer_);
 }
 
 Error HttpFsNode::GetStat_Locked(struct stat* stat) {
@@ -241,7 +253,17 @@ Error HttpFsNode::GetStat_Locked(struct stat* stat) {
     ScopedResource response(filesystem_->ppapi());
     int32_t statuscode;
     StringMap_t response_headers;
-    Error error = OpenUrl("HEAD",
+    const char* method = "HEAD";
+
+    if (filesystem->is_blob_url_) {
+      // Blob URLs do not support HEAD requests, but do give the content length
+      // in their response headers. We issue a single-byte GET request to
+      // retrieve the content length.
+      method = "GET";
+      headers["Range"] = "bytes=0-0";
+    }
+
+    Error error = OpenUrl(method,
                           &headers,
                           &loader,
                           &request,
@@ -252,7 +274,9 @@ Error HttpFsNode::GetStat_Locked(struct stat* stat) {
       return error;
 
     off_t entity_length;
-    if (ParseContentLength(response_headers, &entity_length)) {
+    if (ParseContentRange(response_headers, NULL, NULL, &entity_length)) {
+      SetCachedSize(static_cast<off_t>(entity_length));
+    } else if (ParseContentLength(response_headers, &entity_length)) {
       SetCachedSize(static_cast<off_t>(entity_length));
     } else if (cache_content_) {
       // The server didn't give a content length; download the data to memory
@@ -277,7 +301,7 @@ Error HttpFsNode::GetStat_Locked(struct stat* stat) {
     stat_.st_mtime = 0;
     stat_.st_ctime = 0;
 
-    stat_.st_mode |= S_IFREG;
+    SetType(S_IFREG);
   }
 
   // Fill the stat structure if provided
@@ -320,7 +344,7 @@ Error HttpFsNode::OpenUrl(const char* method,
                                           out_request->pp_resource(),
                                           PP_BlockUntilComplete());
   if (result != PP_OK)
-    return PPErrorToErrno(result);
+    return PPERROR_TO_ERRNO(result);
 
   out_response->Reset(
       loader_interface->GetResponseInfo(out_loader->pp_resource()));
@@ -539,12 +563,18 @@ Error HttpFsNode::ReadEntireResponseToTemp(const ScopedResource& loader,
   *out_bytes = 0;
 
   const int kBytesToRead = MAX_READ_BUFFER_SIZE;
-  buffer_.resize(kBytesToRead);
+  buffer_ = (char*)realloc(buffer_, kBytesToRead);
+  assert(buffer_);
+  if (!buffer_) {
+    buffer_len_ = 0;
+    return ENOMEM;
+  }
+  buffer_len_ = kBytesToRead;
 
   while (true) {
     int bytes_read;
     Error error =
-        ReadResponseToBuffer(loader, buffer_.data(), kBytesToRead, &bytes_read);
+        ReadResponseToBuffer(loader, buffer_, kBytesToRead, &bytes_read);
     if (error)
       return error;
 
@@ -586,16 +616,23 @@ Error HttpFsNode::ReadResponseToTemp(const ScopedResource& loader,
                                      int* out_bytes) {
   *out_bytes = 0;
 
-  if (buffer_.size() < static_cast<size_t>(count))
-    buffer_.resize(std::min(count, MAX_READ_BUFFER_SIZE));
+  if (buffer_len_ < count) {
+    int new_len = std::min(count, MAX_READ_BUFFER_SIZE);
+    buffer_ = (char*)realloc(buffer_, new_len);
+    assert(buffer_);
+    if (!buffer_) {
+      buffer_len_ = 0;
+      return ENOMEM;
+    }
+    buffer_len_ = new_len;
+  }
 
   int bytes_left = count;
   while (bytes_left > 0) {
-    int bytes_to_read =
-        std::min(static_cast<size_t>(bytes_left), buffer_.size());
+    int bytes_to_read = std::min(bytes_left, buffer_len_);
     int bytes_read;
     Error error = ReadResponseToBuffer(
-        loader, buffer_.data(), bytes_to_read, &bytes_read);
+        loader, buffer_, bytes_to_read, &bytes_read);
     if (error)
       return error;
 
@@ -635,7 +672,7 @@ Error HttpFsNode::ReadResponseToBuffer(const ScopedResource& loader,
     }
 
     if (bytes_read < 0)
-      return PPErrorToErrno(bytes_read);
+      return PPERROR_TO_ERRNO(bytes_read);
 
     assert(bytes_read <= bytes_to_read);
     bytes_to_read -= bytes_read;
