@@ -56,8 +56,10 @@ class KernelProxy : protected KernelObject {
   bool RegisterFsType(const char* fs_type, fuse_operations* fuse_ops);
   bool UnregisterFsType(const char* fs_type);
 
-  bool RegisterExitHandler(nacl_io_exit_handler_t exit_handler,
-                           void* user_data);
+  void SetExitCallback(nacl_io_exit_callback_t exit_callback, void* user_data);
+
+  void SetMountCallback(nacl_io_mount_callback_t mount_callback,
+                        void* user_data);
 
   virtual int pipe(int pipefds[2]);
 
@@ -65,7 +67,7 @@ class KernelProxy : protected KernelObject {
   virtual int open_resource(const char* file);
 
   // KernelHandle and FD allocation and manipulation functions.
-  virtual int open(const char* path, int open_flags);
+  virtual int open(const char* path, int open_flags, mode_t mode);
   virtual int close(int fd);
   virtual int dup(int fd);
   virtual int dup2(int fd, int newfd);
@@ -87,7 +89,6 @@ class KernelProxy : protected KernelObject {
   virtual int chown(const char* path, uid_t owner, gid_t group);
   virtual int fchown(int fd, uid_t owner, gid_t group);
   virtual int lchown(const char* path, uid_t owner, gid_t group);
-  virtual int utime(const char* filename, const struct utimbuf* times);
 
   // System calls that take a path as an argument: The kernel proxy will look
   // for the Node associated to the path. To find the node, the kernel proxy
@@ -106,16 +107,17 @@ class KernelProxy : protected KernelObject {
   virtual ssize_t read(int fd, void* buf, size_t nbyte);
   virtual ssize_t write(int fd, const void* buf, size_t nbyte);
 
-  virtual int fchmod(int fd, int prot);
+  virtual int fchmod(int fd, mode_t mode);
   virtual int fcntl(int fd, int request, va_list args);
   virtual int fstat(int fd, struct stat* buf);
-  virtual int getdents(int fd, void* buf, unsigned int count);
+  virtual int getdents(int fd, struct dirent* buf, unsigned int count);
   virtual int fchdir(int fd);
   virtual int ftruncate(int fd, off_t length);
   virtual int fsync(int fd);
   virtual int fdatasync(int fd);
   virtual int isatty(int fd);
   virtual int ioctl(int fd, int request, va_list args);
+  virtual int futimens(int fd, const struct timespec times[2]);
 
   // lseek() relies on the filesystem's Stat() to determine whether or not the
   // file handle corresponding to fd is a directory
@@ -133,7 +135,7 @@ class KernelProxy : protected KernelObject {
   // access() uses the Filesystem's Stat().
   virtual int access(const char* path, int amode);
   virtual int readlink(const char* path, char* buf, size_t count);
-  virtual int utimes(const char* filename, const struct timeval times[2]);
+  virtual int utimens(const char* path, const struct timespec times[2]);
 
   virtual int link(const char* oldpath, const char* newpath);
   virtual int symlink(const char* oldpath, const char* newpath);
@@ -155,6 +157,7 @@ class KernelProxy : protected KernelObject {
   virtual int sigaction(int signum,
                         const struct sigaction* action,
                         struct sigaction* oaction);
+  virtual mode_t umask(mode_t);
 
 #ifdef PROVIDES_SOCKET_API
   virtual int select(int nfds,
@@ -175,6 +178,13 @@ class KernelProxy : protected KernelObject {
                           const char* service,
                           const struct addrinfo* hints,
                           struct addrinfo** res);
+  virtual int getnameinfo(const struct sockaddr *sa,
+                          socklen_t salen,
+                          char *host,
+                          size_t hostlen,
+                          char *serv,
+                          size_t servlen,
+                          int flags);
   virtual int getpeername(int fd, struct sockaddr* addr, socklen_t* len);
   virtual int getsockname(int fd, struct sockaddr* addr, socklen_t* len);
   virtual int getsockopt(int fd,
@@ -190,6 +200,7 @@ class KernelProxy : protected KernelObject {
                            int flags,
                            struct sockaddr* addr,
                            socklen_t* addrlen);
+  // recvmsg ignores ancillary data.
   virtual ssize_t recvmsg(int fd, struct msghdr* msg, int flags);
   virtual ssize_t send(int fd, const void* buf, size_t len, int flags);
   virtual ssize_t sendto(int fd,
@@ -198,6 +209,7 @@ class KernelProxy : protected KernelObject {
                          int flags,
                          const struct sockaddr* addr,
                          socklen_t addrlen);
+  // sendmsg ignores ancillary data.
   virtual ssize_t sendmsg(int fd, const struct msghdr* msg, int flags);
   virtual int setsockopt(int fd,
                          int lvl,
@@ -218,6 +230,9 @@ class KernelProxy : protected KernelObject {
                       bool create_fs_node,
                       ScopedFilesystem* out_filesystem);
 
+  Error FutimensInternal(const ScopedNode& node,
+                         const struct timespec times[2]);
+
   Error CreateFsNode(const ScopedFilesystem& fs);
 
  protected:
@@ -228,8 +243,10 @@ class KernelProxy : protected KernelObject {
   PepperInterface* ppapi_;
   static KernelProxy* s_instance_;
   struct sigaction sigwinch_handler_;
-  nacl_io_exit_handler_t exit_handler_;
-  void* exit_handler_user_data_;
+  nacl_io_exit_callback_t exit_callback_;
+  void* exit_callback_user_data_;
+  nacl_io_mount_callback_t mount_callback_;
+  void* mount_callback_user_data_;
 #ifdef PROVIDES_SOCKET_API
   HostResolver host_resolver_;
 #endif
