@@ -3,12 +3,10 @@
 // found in the LICENSE file.
 
 #include <sys/types.h>  // Include something that will define __GLIBC__.
+#include "nacl_io/kernel_wrap.h" // IRT_EXT is turned on in this header.
 
-// The entire file is wrapped in this #if. We do this so this .cc file can be
-// compiled, even on a non-glibc build.
-#if defined(__native_client__) && defined(__GLIBC__)
-
-#include "nacl_io/kernel_wrap.h"
+#if !defined(NACL_IO_IRT_EXT) && defined(__native_client__) && \
+    defined(__GLIBC__)
 
 #include <alloca.h>
 #include <assert.h>
@@ -24,7 +22,9 @@
 #include "nacl_io/kernel_intercept.h"
 #include "nacl_io/kernel_wrap_real.h"
 #include "nacl_io/log.h"
+#include "nacl_io/nacl_abi_types.h"
 #include "nacl_io/osmman.h"
+#include "nacl_io/ostime.h"
 
 namespace {
 
@@ -41,8 +41,11 @@ void stat_to_nacl_stat(const struct stat* buf, nacl_abi_stat* nacl_buf) {
   nacl_buf->nacl_abi_st_blksize = buf->st_blksize;
   nacl_buf->nacl_abi_st_blocks = buf->st_blocks;
   nacl_buf->nacl_abi_st_atime = buf->st_atime;
+  nacl_buf->nacl_abi_st_atimensec = buf->st_atimensec;
   nacl_buf->nacl_abi_st_mtime = buf->st_mtime;
+  nacl_buf->nacl_abi_st_mtimensec = buf->st_mtimensec;
   nacl_buf->nacl_abi_st_ctime = buf->st_ctime;
+  nacl_buf->nacl_abi_st_ctimensec = buf->st_ctimensec;
 }
 
 void nacl_stat_to_stat(const nacl_abi_stat* nacl_buf, struct stat* buf) {
@@ -58,42 +61,15 @@ void nacl_stat_to_stat(const nacl_abi_stat* nacl_buf, struct stat* buf) {
   buf->st_blksize = nacl_buf->nacl_abi_st_blksize;
   buf->st_blocks = nacl_buf->nacl_abi_st_blocks;
   buf->st_atime = nacl_buf->nacl_abi_st_atime;
+  buf->st_atimensec = nacl_buf->nacl_abi_st_atimensec;
   buf->st_mtime = nacl_buf->nacl_abi_st_mtime;
+  buf->st_mtimensec = nacl_buf->nacl_abi_st_mtimensec;
   buf->st_ctime = nacl_buf->nacl_abi_st_ctime;
+  buf->st_ctimensec = nacl_buf->nacl_abi_st_ctimensec;
 }
 
 }  // namespace
 
-// From native_client/src/trusted/service_runtime/include/sys/dirent.h
-
-#ifndef nacl_abi___ino_t_defined
-#define nacl_abi___ino_t_defined
-typedef int64_t nacl_abi___ino_t;
-typedef nacl_abi___ino_t nacl_abi_ino_t;
-#endif
-
-#ifndef nacl_abi___off_t_defined
-#define nacl_abi___off_t_defined
-typedef int64_t nacl_abi__off_t;
-typedef nacl_abi__off_t nacl_abi_off_t;
-#endif
-
-/* We need a way to define the maximum size of a name. */
-#ifndef MAXNAMLEN
-# ifdef NAME_MAX
-#  define MAXNAMLEN NAME_MAX
-# else
-#  define MAXNAMLEN 255
-# endif
-#endif
-
-struct nacl_abi_dirent {
-  nacl_abi_ino_t nacl_abi_d_ino;
-  nacl_abi_off_t nacl_abi_d_off;
-  uint16_t       nacl_abi_d_reclen;
-  char           nacl_abi_d_name[MAXNAMLEN + 1];
-};
-
 static const int d_name_shift = offsetof (dirent, d_name) -
   offsetof (struct nacl_abi_dirent, nacl_abi_d_name);
 
@@ -106,7 +82,7 @@ EXTERN_C_BEGIN
 #define WRAP(name) __nacl_irt_##name##_wrap
 
 // Declare REAL function pointer.
-#define DECLARE_REAL_PTR(name) typeof(__nacl_irt_##name) REAL(name);
+#define DECLARE_REAL_PTR(name) __typeof__(__nacl_irt_##name) REAL(name);
 
 // Assign the REAL function pointer.
 #define ASSIGN_REAL_PTR(name)        \
@@ -114,10 +90,12 @@ EXTERN_C_BEGIN
   REAL(name) = __nacl_irt_##name;
 
 // Switch IRT's pointer to the REAL pointer
-#define USE_REAL(name) __nacl_irt_##name = (typeof(__nacl_irt_##name))REAL(name)
+#define USE_REAL(name) \
+  __nacl_irt_##name = (__typeof__(__nacl_irt_##name))REAL(name)
 
 // Switch IRT's pointer to the WRAP function
-#define USE_WRAP(name) __nacl_irt_##name = (typeof(__nacl_irt_##name))WRAP(name)
+#define USE_WRAP(name) \
+  __nacl_irt_##name = (__typeof__(__nacl_irt_##name))WRAP(name)
 
 #define EXPAND_SYMBOL_LIST_OPERATION(OP) \
   OP(chdir);                             \
@@ -157,7 +135,20 @@ EXTERN_C_BEGIN
   OP(getsockopt);                        \
   OP(setsockopt);                        \
   OP(socketpair);                        \
-  OP(shutdown);
+  OP(shutdown);                          \
+                                         \
+  OP(chmod);                             \
+  OP(access);                            \
+  OP(unlink);                            \
+  OP(fchdir);                            \
+  OP(fchmod);                            \
+  OP(fsync);                             \
+  OP(fdatasync);                         \
+  OP(lstat);                             \
+  OP(link);                              \
+  OP(rename);                            \
+  OP(readlink);                          \
+  OP(utimes);
 
 // TODO(bradnelson): Add these as well.
 // OP(epoll_create);
@@ -166,18 +157,15 @@ EXTERN_C_BEGIN
 // OP(epoll_pwait);
 // OP(ppoll);
 // OP(pselect);
-//
 
 EXPAND_SYMBOL_LIST_OPERATION(DECLARE_REAL_PTR);
 
 int WRAP(chdir)(const char* pathname) {
-  RTN_ERRNO_IF(ki_chdir(pathname) < 0);
-  return 0;
+  ERRNO_RTN(ki_chdir(pathname));
 }
 
 int WRAP(close)(int fd) {
-  RTN_ERRNO_IF(ki_close(fd) < 0);
-  return 0;
+  ERRNO_RTN(ki_close(fd));
 }
 
 int WRAP(dup)(int fd, int* newfd) NOTHROW {
@@ -187,8 +175,7 @@ int WRAP(dup)(int fd, int* newfd) NOTHROW {
 }
 
 int WRAP(dup2)(int fd, int newfd) NOTHROW {
-  RTN_ERRNO_IF(ki_dup2(fd, newfd) < 0);
-  return 0;
+  ERRNO_RTN(ki_dup2(fd, newfd));
 }
 
 void WRAP(exit)(int status) {
@@ -209,7 +196,8 @@ int WRAP(getcwd)(char* buf, size_t size) {
   return 0;
 }
 
-int WRAP(getdents)(int fd, dirent* nacl_buf, size_t nacl_count, size_t* nread) {
+int WRAP(getdents)(int fd, nacl_abi_dirent* nacl_buf, size_t nacl_count,
+                   size_t* nread) {
   int nacl_offset = 0;
   // "buf" contains dirent(s); "nacl_buf" contains nacl_abi_dirent(s).
   // nacl_abi_dirent(s) are smaller than dirent(s), so nacl_count bytes buffer
@@ -218,7 +206,7 @@ int WRAP(getdents)(int fd, dirent* nacl_buf, size_t nacl_count, size_t* nread) {
   int offset = 0;
   int count;
 
-  count = ki_getdents(fd, buf, nacl_count);
+  count = ki_getdents(fd, (dirent*)buf, nacl_count);
   RTN_ERRNO_IF(count < 0);
 
   while (offset < count) {
@@ -264,8 +252,8 @@ int WRAP(munmap)(void* addr, size_t length) {
   return REAL(munmap)(addr, length);
 }
 
-int WRAP(open)(const char* pathname, int oflag, mode_t cmode, int* newfd) {
-  *newfd = ki_open(pathname, oflag);
+int WRAP(open)(const char* pathname, int oflag, mode_t mode, int* newfd) {
+  *newfd = ki_open(pathname, oflag, mode);
   RTN_ERRNO_IF(*newfd < 0);
   return 0;
 }
@@ -320,6 +308,65 @@ int WRAP(stat)(const char* pathname, struct nacl_abi_stat* nacl_buf) {
   return 0;
 }
 
+int WRAP(lstat)(const char* pathname, struct nacl_abi_stat* nacl_buf) {
+  struct stat buf;
+  memset(&buf, 0, sizeof(struct stat));
+  int res = ki_lstat(pathname, &buf);
+  RTN_ERRNO_IF(res < 0);
+  stat_to_nacl_stat(&buf, nacl_buf);
+  return 0;
+}
+
+int WRAP(link)(const char* pathname, const char* newpath) {
+  ERRNO_RTN(ki_link(pathname, newpath));
+}
+
+int WRAP(rename)(const char* pathname, const char* newpath) {
+  ERRNO_RTN(ki_rename(pathname, newpath));
+}
+
+int WRAP(readlink)(const char* pathname,
+                   char* buf,
+                   size_t count,
+                   size_t* nread) {
+  int rtn = ki_readlink(pathname, buf, count);
+  RTN_ERRNO_IF(rtn < 0);
+  *nread = rtn;
+  return 0;
+}
+
+int WRAP(utimes)(const char *filename, const struct timeval *times) {
+  ERRNO_RTN(ki_utimes(filename, times));
+}
+
+int WRAP(chmod)(const char* pathname, mode_t mode) {
+  ERRNO_RTN(ki_chmod(pathname, mode));
+}
+
+int WRAP(access)(const char* pathname, int amode) {
+  ERRNO_RTN(ki_access(pathname, amode));
+}
+
+int WRAP(unlink)(const char* pathname) {
+  ERRNO_RTN(ki_unlink(pathname));
+}
+
+int WRAP(fchdir)(int fd) {
+  ERRNO_RTN(ki_fchdir(fd));
+}
+
+int WRAP(fchmod)(int fd, mode_t mode) {
+  ERRNO_RTN(ki_fchmod(fd, mode));
+}
+
+int WRAP(fsync)(int fd) {
+  ERRNO_RTN(ki_fsync(fd));
+}
+
+int WRAP(fdatasync)(int fd) {
+  ERRNO_RTN(ki_fdatasync(fd));
+}
+
 int WRAP(write)(int fd, const void* buf, size_t count, size_t* nwrote) {
   ssize_t signed_nwrote = ki_write(fd, buf, count);
   *nwrote = static_cast<size_t>(signed_nwrote);
@@ -337,23 +384,19 @@ int WRAP(accept)(int sockfd,
 }
 
 int WRAP(bind)(int sockfd, const struct sockaddr* addr, socklen_t addrlen) {
-  RTN_ERRNO_IF(ki_bind(sockfd, addr, addrlen) < 0);
-  return 0;
+  ERRNO_RTN(ki_bind(sockfd, addr, addrlen));
 }
 
 int WRAP(connect)(int sockfd, const struct sockaddr* addr, socklen_t addrlen) {
-  RTN_ERRNO_IF(ki_connect(sockfd, addr, addrlen) < 0);
-  return 0;
+  ERRNO_RTN(ki_connect(sockfd, addr, addrlen));
 }
 
 int WRAP(getpeername)(int sockfd, struct sockaddr* addr, socklen_t* addrlen) {
-  RTN_ERRNO_IF(ki_getpeername(sockfd, addr, addrlen) < 0);
-  return 0;
+  ERRNO_RTN(ki_getpeername(sockfd, addr, addrlen));
 }
 
 int WRAP(getsockname)(int sockfd, struct sockaddr* addr, socklen_t* addrlen) {
-  RTN_ERRNO_IF(ki_getsockname(sockfd, addr, addrlen) < 0);
-  return 0;
+  ERRNO_RTN(ki_getsockname(sockfd, addr, addrlen));
 }
 
 int WRAP(getsockopt)(int sockfd,
@@ -361,8 +404,7 @@ int WRAP(getsockopt)(int sockfd,
                      int optname,
                      void* optval,
                      socklen_t* optlen) {
-  RTN_ERRNO_IF(ki_getsockopt(sockfd, level, optname, optval, optlen) < 0);
-  return 0;
+  ERRNO_RTN(ki_getsockopt(sockfd, level, optname, optval, optlen));
 }
 
 int WRAP(setsockopt)(int sockfd,
@@ -370,13 +412,11 @@ int WRAP(setsockopt)(int sockfd,
                      int optname,
                      const void* optval,
                      socklen_t optlen) {
-  RTN_ERRNO_IF(ki_setsockopt(sockfd, level, optname, optval, optlen) < 0);
-  return 0;
+  ERRNO_RTN(ki_setsockopt(sockfd, level, optname, optval, optlen));
 }
 
 int WRAP(listen)(int sockfd, int backlog) {
-  RTN_ERRNO_IF(ki_listen(sockfd, backlog) < 0);
-  return 0;
+  ERRNO_RTN(ki_listen(sockfd, backlog));
 }
 
 int WRAP(recv)(int sockfd, void* buf, size_t len, int flags, int* count) {
@@ -461,9 +501,12 @@ static void assign_real_pointers() {
   }
 }
 
-#define CHECK_REAL(func) \
-  if (!REAL(func))       \
-    assign_real_pointers();
+#define CHECK_REAL(func)    \
+  if (!REAL(func)) {        \
+    assign_real_pointers(); \
+    if (!REAL(func))        \
+      return ENOSYS;        \
+  }
 
 // "real" functions, i.e. the unwrapped original functions.
 
@@ -473,7 +516,8 @@ int _real_close(int fd) {
 }
 
 void _real_exit(int status) {
-  CHECK_REAL(exit);
+  if (!REAL(exit))
+    assign_real_pointers();
   REAL(exit)(status);
 }
 
@@ -545,9 +589,9 @@ int _real_munmap(void* addr, size_t length) {
   return REAL(munmap)(addr, length);
 }
 
-int _real_open(const char* pathname, int oflag, mode_t cmode, int* newfd) {
+int _real_open(const char* pathname, int oflag, mode_t mode, int* newfd) {
   CHECK_REAL(open);
-  return REAL(open)(pathname, oflag, cmode, newfd);
+  return REAL(open)(pathname, oflag, mode, newfd);
 }
 
 int _real_open_resource(const char* file, int* fd) {
@@ -570,6 +614,11 @@ int _real_write(int fd, const void* buf, size_t count, size_t* nwrote) {
   return REAL(write)(fd, buf, count, nwrote);
 }
 
+int _real_getcwd(char* pathname, size_t len) {
+  CHECK_REAL(getcwd);
+  return REAL(getcwd)(pathname, len);
+}
+
 static bool s_wrapped = false;
 void kernel_wrap_init() {
   if (!s_wrapped) {
