@@ -3,12 +3,12 @@
 // found in the LICENSE file.
 
 #include <sys/types.h>  // Include something that will define __GLIBC__.
+#include "nacl_io/kernel_wrap.h" // IRT_EXT is turned on in this header.
 
 // The entire file is wrapped in this #if. We do this so this .cc file can be
 // compiled, even on a non-newlib build.
-#if defined(__native_client__) && !defined(__GLIBC__) && !defined(__BIONIC__)
-
-#include "nacl_io/kernel_wrap.h"
+#if !defined(NACL_IO_IRT_EXT) && defined(__native_client__) && \
+    !defined(__GLIBC__) && !defined(__BIONIC__)
 
 #include <dirent.h>
 #include <errno.h>
@@ -32,20 +32,21 @@ EXTERN_C_BEGIN
 
 // Declare REAL function pointer.
 #define DECLARE_REAL_PTR(group, name) \
-  typeof(__libnacl_irt_##group.name) REAL(name);
+  __typeof__(__libnacl_irt_##group.name) REAL(name);
 
 // Assign the REAL function pointer.
 #define ASSIGN_REAL_PTR(group, name) REAL(name) = __libnacl_irt_##group.name;
 
 // Switch IRT's pointer to the REAL pointer
 #define USE_REAL(group, name) \
-  __libnacl_irt_##group.name = (typeof(REAL(name)))REAL(name);
+  __libnacl_irt_##group.name = (__typeof__(REAL(name)))REAL(name);
 
 // Switch the IRT's pointer to the WRAP function
 #define USE_WRAP(group, name) \
-  __libnacl_irt_##group.name = (typeof(REAL(name)))WRAP(name);
+  __libnacl_irt_##group.name = (__typeof__(REAL(name)))WRAP(name);
 
 extern void __libnacl_irt_dev_filename_init(void);
+extern void __libnacl_irt_dev_fdio_init(void);
 
 extern struct nacl_irt_basic __libnacl_irt_basic;
 extern struct nacl_irt_fdio __libnacl_irt_fdio;
@@ -184,8 +185,8 @@ int WRAP(munmap)(void* addr, size_t length) {
   return REAL(munmap)(addr, length);
 }
 
-int WRAP(open)(const char* pathname, int oflag, mode_t cmode, int* newfd) {
-  *newfd = ki_open(pathname, oflag);
+int WRAP(open)(const char* pathname, int oflag, mode_t mode, int* newfd) {
+  *newfd = ki_open(pathname, oflag, mode);
   ERRNO_RTN(*newfd);
 }
 
@@ -261,14 +262,18 @@ static void assign_real_pointers() {
   static bool assigned = false;
   if (!assigned) {
     __libnacl_irt_dev_filename_init();
+    __libnacl_irt_dev_fdio_init();
     EXPAND_SYMBOL_LIST_OPERATION(ASSIGN_REAL_PTR)
     assigned = true;
   }
 }
 
-#define CHECK_REAL(func) \
-  if (!REAL(func))       \
-    assign_real_pointers();
+#define CHECK_REAL(func)    \
+  if (!REAL(func)) {        \
+    assign_real_pointers(); \
+    if (!REAL(func))        \
+      return ENOSYS;        \
+  }
 
 // "real" functions, i.e. the unwrapped original functions.
 
@@ -278,7 +283,8 @@ int _real_close(int fd) {
 }
 
 void _real_exit(int status) {
-  CHECK_REAL(exit);
+  if (!REAL(exit))
+    assign_real_pointers();
   REAL(exit)(status);
 }
 
@@ -289,6 +295,12 @@ int _real_fstat(int fd, struct stat* buf) {
 
 int _real_isatty(int fd, int* result) {
   CHECK_REAL(isatty);
+  // The real isatty function can be NULL (for example if we are running
+  // within chrome).
+  if (REAL(isatty) == NULL) {
+    *result = 0;
+    return ENOTTY;
+  }
   return REAL(isatty)(fd, result);
 }
 
@@ -303,7 +315,8 @@ int _real_lseek(int fd, off_t offset, int whence, off_t* new_offset) {
 }
 
 int _real_mkdir(const char* pathname, mode_t mode) {
-  return ENOSYS;
+  CHECK_REAL(mkdir);
+  return REAL(mkdir)(pathname, mode);
 }
 
 int _real_mmap(void** addr,
@@ -321,9 +334,9 @@ int _real_munmap(void* addr, size_t length) {
   return REAL(munmap)(addr, length);
 }
 
-int _real_open(const char* pathname, int oflag, mode_t cmode, int* newfd) {
+int _real_open(const char* pathname, int oflag, mode_t mode, int* newfd) {
   CHECK_REAL(open);
-  return REAL(open)(pathname, oflag, cmode, newfd);
+  return REAL(open)(pathname, oflag, mode, newfd);
 }
 
 int _real_open_resource(const char* file, int* fd) {
@@ -336,7 +349,8 @@ int _real_read(int fd, void* buf, size_t count, size_t* nread) {
 }
 
 int _real_rmdir(const char* pathname) {
-  return ENOSYS;
+  CHECK_REAL(rmdir);
+  return REAL(rmdir)(pathname);
 }
 
 int _real_write(int fd, const void* buf, size_t count, size_t* nwrote) {
@@ -344,6 +358,11 @@ int _real_write(int fd, const void* buf, size_t count, size_t* nwrote) {
   return REAL(write)(fd, buf, count, nwrote);
 }
 
+int _real_getcwd(char* pathname, size_t len) {
+  CHECK_REAL(getcwd);
+  return REAL(getcwd)(pathname, len);
+}
+
 static bool s_wrapped = false;
 
 void kernel_wrap_init() {
